/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
#include "usb_lib.h"
#include "usb_mem.h"
#include "TargetMonitorProtocol.h"
#include <string.h>

extern u32  count_in;
extern u32  count_out;
extern u8 buffer_out[];

/*******************************************************************************
* Function Name  : sendBlock.
* Description    : Send a data block through the USB port
* Input          : Data pointer and length.
* Output         : None.
* Return         : None.
*******************************************************************************/
void sendBlock(u8	* pData,	u8	length)
{
	if (bDeviceState == CONFIGURED)
	{
	  count_in = length;
	  UserToPMABufferCopy(pData, ENDP1_TXADDR, length);
	  SetEPTxCount(ENDP1, length);
	  SetEPTxValid(ENDP1);
	}
}


/*******************************************************************************
* Function Name  : receiveBlock.
* Description    : Receive a data block through the USB port
* Input          : Data pointer and maximum length.
* Output         : None.
* Return         : Actual length.
*******************************************************************************/
u8 receiveBlock(u8	* pData,	u8	maxLen)
{
	if (count_out > 0)
	{
		if (maxLen < count_out)
		{
			/* Requested less than available, so copy data and move remainder up */
			count_out-= maxLen;
			memcpy(pData, buffer_out, maxLen);
			memcpy(buffer_out, &buffer_out[maxLen], count_out);
		}
		else
		{
			maxLen = count_out;
			memcpy(pData, buffer_out, maxLen);
			count_out = 0;
		}
	}
	else
	{
		maxLen = 0;
	}

	return maxLen;
}

/*******************************************************************************
* Function Name  : sendAvailable.
* Description    : Indicate if the output buffer is available
* Input          : None.
* Output         : None.
* Return         : TRUE if send is possible
*******************************************************************************/
u8 sendAvailable(void)
{
	return (count_in == 0);
}


/*********** Use USB port on STM32 Circle **************/
int abortComm = 0;

#define VIRTUAL_COM_PORT_DATA_SIZE              64
#define VIRTUAL_COM_PORT_INT_SIZE               8

static unsigned char USBReceiveBlock(char * pBuffer, unsigned char length)
{
	unsigned char	curLen = 0;
	unsigned char	remainder = 0;
	unsigned char	newLen= 0;
	unsigned char	blockLen= 0;

	// The most we can receive in one go without an ack is VIRTUAL_COM_PORT_DATA_SIZE
	blockLen = length;
	if (blockLen > VIRTUAL_COM_PORT_DATA_SIZE)
	{
		blockLen = VIRTUAL_COM_PORT_DATA_SIZE;
	}

	// Receive a specified amount of data, returns either 0 if none available, or specified length
	curLen = receiveBlock(pBuffer, blockLen);

	if (curLen > 0)
	{
		// We have received some, do we need more
		while (blockLen > 0)
		{
			// We got some, did we get all?
			remainder = blockLen - curLen;
			while (remainder > 0)
			{
				// No so we must wait for the rest
				newLen = receiveBlock(&pBuffer[curLen], remainder);

				curLen+= newLen;
				remainder-= newLen;
			}

			// Have we received all?
			if (blockLen == length)
			{
				// Yes so leave ack to caller
				break;
			}

			// We have a partial block, send an ack and wait again
			sendUSBAck();

			length-= blockLen;
			blockLen = length;
			if (blockLen > VIRTUAL_COM_PORT_DATA_SIZE)
			{
				blockLen = VIRTUAL_COM_PORT_DATA_SIZE;
			}
		}
	}

	return curLen;
}

void awaitUSBConnection()
{
	static const char	  * connectMsg = "USBConnecting";
	static const char	  * responseMsg = "USBConnected";
	const unsigned char				connectLen = 13;
	const unsigned char				responseLen = 12;
	long					start;
	unsigned char						rxBuffer[32];
	unsigned char						rxLen;
	int				loopCount;
	unsigned char						connected = FALSE;

	// Loop sending connect messages to the host until we get an answer
	while (!connected)
	{
		sendBlock((char *) connectMsg, connectLen);

		loopCount = 0;

		while (loopCount < 1000)
		{
			rxLen = USBReceiveBlock(rxBuffer, responseLen);

			if (rxLen == responseLen)
			{
				if (strncmp(rxBuffer, responseMsg, responseLen) == 0)
				{
					// We are connected!
					connected = TRUE;
					break;
				}
			}

		{	// Wait for the next system tick to occur (1ms)
        int yyy = 0;
      	while (yyy<100)
				{yyy++;}
			///sleep = RiCTRUE;
      }
			loopCount++;
		}
	}
}


/*****************/
void stm32_USB_InitProtocol()
{
	// We need to initialise the whole BSP, as this is called before the framework Init
	Device_Init();

	// Now wait for the host to connect
	awaitUSBConnection();
	/* since we waited for connection, the timer had ran for some time
		we want to start the app as if on time 0 */
	RiCTimerManager_resetSystemTimerElapsedTime();

	return;
}
/*********/


/************** End USB on STM32 Circle ***********/
