/*

	Licensed Materials - Property of IBM
	(c) Copyright IBM Corporation 2006, 2011. All Rights Reserved.
*/

/********************************************************************
 ---------------------------------------------------------------
|							  RiCEventQueue.c                          |
 --------------------------------------------------------------- 
*********************************************************************/
#ifndef MXF_SOURCE
#define MXF_SOURCE
#endif
#ifndef MXF_RiCEventQueue_SOURCE
#define MXF_RiCEventQueue_SOURCE
#endif

#include <stdlib.h>
#include "RiCEventQueue.h"

/***************************************************/
/****			 Private operations		        ****/
/***************************************************/
static RiCBoolean isFull_(const RiCEventQueue * const me);
static void increaseHead_(RiCEventQueue * const me);
static void increaseTail_(RiCEventQueue * const me);

/** Internal check of Queue occupation status **/
static RiCBoolean isFull_(const RiCEventQueue * const me)
{
	RiCBoolean res = RiCFALSE;
	if ((NULL != me) && (me->count == me->size)){
		res = RiCTRUE;
	}
	return res;
}

/** Move the queue head to a new position, after removing the top **/
static void increaseHead_(RiCEventQueue * const me)
{
	if ((NULL != me) && (RiCEventQueue_isEmpty(me) == RiCFALSE)){
		(me->head)++;
		if ((me->head) == (me->size)){
			me->head = 0U;
		}
	}
}

/** Move the queue tail after adding a new element **/
static void increaseTail_(RiCEventQueue * const me)
{
	/* handle full queue */
	if ((NULL != me) && (isFull_(me) == RiCFALSE)) {
		/* queue not empty */
		++(me->tail);
		if ((me->tail == me->size) && (me->head > 0U)){
			me->tail = 0U;
		}
	}
}



/***************************************************/
/****          Creation and Destruction         ****/
/***************************************************/
RiCBoolean RiCEventQueue_Init(RiCEventQueue * const me, RiCEvent* * const evQueue, RhpPositive queueSize)
{
	if (NULL != me) {
		me->tail = 0U;
		me->count = 0U;
		me->size = queueSize;
		me->the_queue = evQueue;
	}
	return RiCTRUE;
}

/*******************/
/*******************/
#ifdef MICROC_OXF_CLEANUP
void RiCEventQueue_Cleanup(RiCEventQueue * const me)
{
	if (NULL != me) {
		me->head = 0;
		me->tail = 0;
		me->count = 0;
	}
}
#endif


/***************************************************/
/****            EventQueue methods             ****/
/***************************************************/

RiCBoolean RiCEventQueue_push(RiCEventQueue * const me, RiCEvent* const pItem)
{
	RiCBoolean res = RiCFALSE;
	if ((NULL != me) && (isFull_(me) == RiCFALSE)){
		me->the_queue[me->tail] = pItem;
		increaseTail_(me);
		me->count++;
		res = RiCTRUE;
	}

#ifdef USE_ASSERT
	RiC_ASSERT(res != RiCFALSE);
#endif

	return res;
}

RiCEvent* RiCEventQueue_pop(RiCEventQueue * const me)
{
	RiCEvent* res = NULL;
	RiCEvent* tmp;

	if ((NULL != me) && (RiCEventQueue_isEmpty(me) == RiCFALSE)){
		tmp = me->the_queue[me->head];
		increaseHead_(me);
		me->count--;
		if(tmp != NULL){
			res = tmp;
		}
	}

#ifdef USE_ASSERT
	RiC_ASSERT(res != NULL);
#endif

	/*LDRA_INSPECTED 71 S */
	return res;
}

#ifdef RIC_USE_RICLIST
/*******************/
/*******************/
void RiCEventQueue_getQueue(const RiCEventQueue * const me, RiCList * const pList)
{
	RhpPositive last, j;

	/* reset the list */
	RiCList_removeAll(pList);
	if ((NULL != me) && (RiCEventQueue_isEmpty(me) == RiCFALSE)){
		last = (me->tail > me->head) ? me->tail : me->size;

		/* copy first part */
		j = me->head;
		for (; j < last; j++) {
			if (me->the_queue[j] != NULL){
				(void) RiCList_addTail(pList, me->the_queue[j]);
			}
		}

		/* check if need to continue */
		if (last != me->tail){
			/* copy second part */
			j = 0U;
			for (; j < me->tail; j++) {
				(void) RiCList_addTail(pList, me->the_queue[j]);
			}
		}
	}
}
#endif

RiCBoolean RiCEventQueue_isEmpty(const RiCEventQueue * const me)
{
	RiCBoolean res = RiCFALSE;
	if ((NULL != me) && (me->count == 0U)){
		res = RiCTRUE;
	}

	return res;
}

RiCBoolean RiCEventQueue_isFull(const RiCEventQueue * const me)
{
	return isFull_(me);
}

