/*

	purpose:	 	Static memory allocation (C version)

	Licensed Materials - Property of IBM
	(c) Copyright IBM Corporation 1999, 2011. All Rights Reserved.



 ---------------------------------------------------------------
|								MemAlloc.c	                      |
 --------------------------------------------------------------- 
*/
#ifndef MXF_SOURCE
#define MXF_SOURCE
#endif
#ifndef MXF_MEMALLOC_SOURCE
#define MXF_MEMALLOC_SOURCE
#endif

#include "MemAlloc.h"

#ifdef RIC_USE_MEMALLOC

STATIC void lockRiCMemoryAllocator(const RiCMemoryAllocator *const me );
STATIC void freeRiCMemoryAllocator(const RiCMemoryAllocator *const me );

/* Functions ******************************************************************/

/*+*****************************************************************************
 * Input:     None.
 *
 * Output:    None.
 *
 * InOut:     me: The memory allocator to lock in order to enter a critical
 *                section.
 *
 * Function:  Entrance to a critical section.
 *
 ****************************************************************************-*/
STATIC void lockRiCMemoryAllocator(const RiCMemoryAllocator *const me )
{
    if((me != NULL) && ((me->isProtected_) != RiCFALSE))
    {
        (void)RiCOSMutex_lock( &(me->myMutex_) );
    }
}


/*+*****************************************************************************
 * Input:     None.
 *
 * Output:    None.
 *
 * InOut:     me: The memory allocator to free in order to exit a critical
 *                section.
 *
 * Function:  Exit from a critical section.
 *
 ****************************************************************************-*/
STATIC void freeRiCMemoryAllocator(const RiCMemoryAllocator *const me )
{
    if((me != NULL) && ((me->isProtected_) != RiCFALSE))
    {
        (void)RiCOSMutex_free( &(me->myMutex_) );
    }
}


/*+*****************************************************************************
  * Input:     memoryPool: The static memory pool for this allocator that holds
 *                        a predefined number of instances.
 *
 *            objectSize: The size of a single instance.
 *
 *            initNum: The number of instances initialy allocated.
 *
 *            incrementNum: The amount (in instance number) to increment if the
 *                          initial pool is exuasted.
 *
 *            isProtected: If true, guard allocator functions to support multi-
 *                         threaded access.
 *
 *            getMemoryPoolNextChunk: Function to get the next empty memory
 *                                    location
 *
 *            setMemoryPoolNextChunk: Function to set the next empty memory
 *                                    location
 *
 *            MemoryPoolIsEmptyClbk: Callback function if memory pool is empty.
 *
 * Output:    None.
 *
 * InOut:     me: The memory allocator to initialize.
 *
 * Function:  Memory allocator initialization.
 *
 ****************************************************************************-*/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
void RiCMemoryAllocator_init( RiCMemoryAllocator * const me,
                              RiC_UINT8 * const memoryPool,
                              const size_t objectSize,
                              const RhpPositive initNum,
                              const RhpPositive incrementNum,
                              const RiCBoolean isProtected,
                              RhpAddress (*const getMemoryPoolNextChunkP)
                                    ( RhpAddress const me ),
                              void(*const setMemoryPoolNextChunkP)
                                  ( RhpAddress me, RhpAddress nextChunk ),
                              void(*const MemoryPoolIsEmptyClbkP) ( void ) )
{
    if( me != NULL )
    {
        me->initialBlock_ = memoryPool;
        me->objectSize_ = objectSize;
        me->initialNum_ = initNum;
        me->incrementNum_ = incrementNum;
        me->isProtected_ = isProtected;
        me->memoryPoolIsEmptyFlag_ = RiCTRUE;
        me->allocator_ = NULL;
        me->getMemoryPoolNextChunk = getMemoryPoolNextChunkP;
        me->setMemoryPoolNextChunk = setMemoryPoolNextChunkP;
        me->MemoryPoolIsEmptyClbk  = MemoryPoolIsEmptyClbkP;

        if(RiCOSMutex_init(&(me->myMutex_), (OSMutexHandle)0) != RiCFALSE)
        {
            lockRiCMemoryAllocator( me );
            {
				/* Justification: Violation of Misra98 Rule45 / Misra04 Rule11.4				*/
				/* Cast to void* pointer is required when using the memory allocation utility */
			 	/*LDRA_INSPECTED 95 S */ /*LDRA_INSPECTED 203 S */
				const RhpAddress newBlock = (RhpAddress) (me->initialBlock_);
				/*LDRA_INSPECTED 98 S : LDRA Analysis incorrect. Actual and formal parameters consistent */ /*LDRA_INSPECTED 458 S */ 
                (void) RiCMemoryAllocator_initiatePool(me, newBlock, me->initialNum_);
            }
            freeRiCMemoryAllocator( me );
        }
    }
}


/*+*****************************************************************************
 * Input:     None.
 *
 * Output:    None.
 *
 * InOut:     me: The memory allocator to cleanup.
 *
 * Function:  Memory allocator cleanup.
 *
 ****************************************************************************-*/
void RiCMemoryAllocator_cleanup( const RiCMemoryAllocator *const me )
{
#ifdef MICROC_OXF_CLEANUP
    if( me != NULL )
    {
        RiCOSMutex_cleanup( &(me->myMutex_) );
    }
#endif
}

/*+*****************************************************************************
 * Input:     size: The size of the memory block to retrieve.
 *
 * Output:    RiCMemoryAllocator_getMemory(): The pointer to the memory block or
 *                                            NULL if failure.
 *
 * InOut:     me: The memory allocator from which the memory block is retrieved.
 *
 * Function:  Retrieve memory from the static memory pool.
 *
 ****************************************************************************-*/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
RhpAddress RiCMemoryAllocator_getMemory( RiCMemoryAllocator *const me,
                                    const size_t objSize )
{
    RhpAddress result = NULL;

    if( me != NULL )
    {
        if( (objSize != 0U) && (objSize != ( me->objectSize_) ) )
        {
#ifdef RIC_USE_DYNAMIC_MEMORY_ALLOCATION
            result = (RhpAddress) RiCGetMemory( objSize );
#else
            result = NULL;
#endif
        }
        else
        {
            lockRiCMemoryAllocator( me );
            {
                result = me->headOfFreeList_;

                /* if result is not valid, allocate more space */
                if( result == NULL )
                {
                    RhpAddress newBlock = NULL;
                    /* out of memory */
                    if((me->memoryPoolIsEmptyFlag_) != RiCFALSE)
                    {
						/*LDRA_INSPECTED 19 D : LDRA Analysis incorrect. Function Pointer call has no available function definition  */
						/*LDRA_INSPECTED 41 D : LDRA Analysis incorrect. Function Pointer call has no available function declaration */	/*LDRA_INSPECTED 496 S */
                        me->MemoryPoolIsEmptyClbk(); /* notification */
                    }
                    if( me->allocator_ != NULL )
                    {
						/*LDRA_INSPECTED 19 D : LDRA Analysis incorrect. Function Pointer call has no available function definition  */
						/*LDRA_INSPECTED 41 D : LDRA Analysis incorrect. Function Pointer call has no available function declaration */	/*LDRA_INSPECTED 496 S */
                        newBlock = me->allocator_( me->incrementNum_ );
                    }
                    else
                    {
                        newBlock = RiCMemoryAllocator_allocPool( me,
                                                              me->incrementNum_,
                                                              me->objectSize_ );
                    }
                    /* connect the pool */
					/*LDRA_INSPECTED 98 S : LDRA Analysis incorrect. Actual and formal parameters consistent */ /*LDRA_INSPECTED 458 S */
					/*LDRA_INSPECTED 45 D : Pointer is checked for null within function */
                    (void)RiCMemoryAllocator_initiatePool(me, newBlock, me->incrementNum_);
                    result = me->headOfFreeList_;
                }

                /* If result is valid, just move the list head to the next
                   element in the free list */
                if( result != NULL )
                {
					/*LDRA_INSPECTED 19 D : LDRA Analysis incorrect. Function Pointer call has no available function definition  */
					/*LDRA_INSPECTED 41 D : LDRA Analysis incorrect. Function Pointer call has no available function declaration */	/*LDRA_INSPECTED 496 S */
                    me->headOfFreeList_ = me->getMemoryPoolNextChunk( result );
                }
            }
            freeRiCMemoryAllocator( me );
        }
    }

#ifdef USE_ASSERT
	RiC_ASSERT(result != NULL);
#endif

	/*LDRA_INSPECTED 71 S */
    return result;
/*LDRA_INSPECTED 5 D */
}


/*+*****************************************************************************
 * Input:     deadObject: The pointer to the memory block to return.
 *
 *            size: The size of the memory block to return.
 *
 * Output:    None.
 *
 * InOut:     me: The memory allocator to which the memory block is returned.
 *
 * Function:  Release used memory.  Passing a memory chunk, which, if it's the
 *            right size, is just added to the front of the list of free chunks.
 *
 ****************************************************************************-*/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
void RiCMemoryAllocator_returnMemory( RiCMemoryAllocator * const me,
                                      RhpAddress const deadObject,
                                      const size_t blockSize )
{
    if( (me != NULL ) && (deadObject != NULL ) )
    {
        /* send objects of "wrong" size to free() */
        if( (blockSize != 0U) && (blockSize != ( me->objectSize_) ) )
        {
			/*LDRA_INSPECTED 98 S : LDRA Analysis incorrect. Actual and formal parameters consistent */ /*LDRA_INSPECTED 458 S */
            RiCReturnMemory( deadObject );
        }
        else
        {
            lockRiCMemoryAllocator( me );
            {
				/*LDRA_INSPECTED 19 D : LDRA Analysis incorrect. Function Pointer call has no available function definition  */
				/*LDRA_INSPECTED 41 D : LDRA Analysis incorrect. Function Pointer call has no available function declaration */	/*LDRA_INSPECTED 496 S */
                me->setMemoryPoolNextChunk( deadObject, me->headOfFreeList_ );
                me->headOfFreeList_ = deadObject;

            }
            freeRiCMemoryAllocator( me );
        }
    }
}


/*+*****************************************************************************
 * Input:     numOfInstances: The number of instances to allocate.
 *
 *            size: The size of a single instance.
 *
 * Output:    RiCMemoryAllocator_allocPool(): The pointer to the new memory
 *                                            pool or NULL on failure.
 *
 * InOut:     me: The memory allocator to which the memory pool is allocated.
 *
 * Function:  Allocate the static memory pool.  This function is NOT protected.
 *            The protection should be done by its callers.
 *
 ****************************************************************************-*/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
RhpAddress RiCMemoryAllocator_allocPool( const RiCMemoryAllocator *const me,
                                    const RhpPositive numOfInstances,
                                    size_t instSize )
{
    /* allocate a big block of memory */
    RhpAddress result = NULL;

    if( me != NULL )
    {
        /* check that  the object size is correct */
        if( instSize != ( me->objectSize_ ) )
        {
            result = RiCGetMemory( instSize );
        }

        else if( numOfInstances > 0U )
        {
#ifdef USE_DYNAMIC_MEMORY_ALLOCATION
            /* The initial pool was used so we will have to allocate another
               chunk using RiCGetMemory().  Still, we benefit by minimizing
               fregmentation and lowering the overhead of many dynamic
               allocations. */
            result = RiCGetMemory( ((size_t) numOfInstances) * me->objectSize_ );
#endif
        }

        else
        {
            /* Do Nothing */
        }
    }
#ifdef USE_ASSERT
	RiC_ASSERT(result != NULL);
#endif

	/*LDRA_INSPECTED 71 S */
    return result;
}


/*+*****************************************************************************
 * Input:     newBlock: Pointer to the first block in the pool.
 *
 *            numOfInstances: The number of instances to allocate in the pool.
 *
 * Output:    RiCMemoryAllocator_initiatePool(): The number of instances that
 *                                               were allocated or 0 on failure.
 *
 * InOut:     me: The memory allocator that will hold the memory pool to
 *                initiate.
 *
 * Function:  Initializes a memory pool.
 *
 ****************************************************************************-*/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used      */
/* Outside of the OXF itself by Generated code				     */
RiC_INT32 RiCMemoryAllocator_initiatePool( RiCMemoryAllocator *const me,
                                           const RhpAddress newBlock,
                                           const RhpPositive numOfInstances )
{
    RiC_INT32 result = 0;
    RiC_UINT32 i_idx;
    /* Justification: Violation of Misra98 Rule45 / Misra04 Rule11.4				*/
	/* Cast from void* pointer is required when using the memory allocation utility */
	/*LDRA_INSPECTED 95 S */ /*LDRA_INSPECTED 203 S */
    const RiC_UINT8 *base_addr = (const RiC_UINT8*) newBlock;

    if( me != NULL )
    {
        /* make sure we got one */
        if( (base_addr == NULL) || (numOfInstances == 0U) )
        {
            me->headOfFreeList_ = NULL;
        }
        else
        {
            /* link the objects together */
            for( i_idx = 0U; i_idx < ( numOfInstances - 1U ); i_idx++ )
            {
		    	/* Justification: Violation of Misra98 Rule45 / Misra04 Rule11.4			  */
				/* Cast to void* pointer is required when using the memory allocation utility */
				/*LDRA_INSPECTED 94 S */
				/*LDRA_INSPECTED 94 S */
				/*LDRA_INSPECTED 97 S : LDRA Analysis incorrect. Casting is required */
 				/*LDRA_INSPECTED 97 S : LDRA Analysis incorrect. Casting is required */
				/*LDRA_INSPECTED 19 D : LDRA Analysis incorrect. Function Pointer call has no available function definition  */
				/*LDRA_INSPECTED 41 D : LDRA Analysis incorrect. Function Pointer call has no available function declaration */	/*LDRA_INSPECTED 496 S */
			    me->setMemoryPoolNextChunk( ((RhpAddress) &base_addr[((size_t) i_idx) * me->objectSize_]),
                                            (RhpAddress) &base_addr[((size_t) ( i_idx + 1U )) * me->objectSize_] );
            }

            /* terminate the linked list with a null pointer */
		    /* Justification: Violation of Misra98 Rule45 / Misra04 Rule11.4			  */
			/* Cast to void* pointer is required when using the memory allocation utility */
			/*LDRA_INSPECTED 94 S */
			/*LDRA_INSPECTED 97 S : LDRA Analysis incorrect. Casting is required */
			/*LDRA_INSPECTED 19 D : LDRA Analysis incorrect. Function Pointer call has no available function definition  */
			/*LDRA_INSPECTED 41 D : LDRA Analysis incorrect. Function Pointer call has no available function declaration */	/*LDRA_INSPECTED 496 S */
            me->setMemoryPoolNextChunk( ((RhpAddress) &base_addr[((size_t) (numOfInstances - 1U)) * me->objectSize_]), NULL );

	        /* Justification: Violation of Misra98 Rule45 / Misra04 Rule11.4			  */
			/* Cast to void* pointer is required when using the memory allocation utility */
            /* set headOfFreeList_ to the first element */
			/*LDRA_INSPECTED 94 S */
            me->headOfFreeList_ = (RhpAddress) &base_addr[0];
            result = 1;
        }
    }

    return result;
}


/*+*****************************************************************************
 * Input:     value: the amount (in instance number) to increment if initial
 *                   pool is exuasted.
 *
 * Output:    None.
 *
 * InOut:     me: The memory allocator whose memory pool shall be incremented
 *                when its initial pool is exhausted.
 *
 * Function:  Set the amount (in instance number) to increment if initial pool
 *            is exuasted.
 *
 ****************************************************************************-*/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
void RiCMemoryAllocator_setIncrementNum( RiCMemoryAllocator *const me,
                                         const RhpPositive theValue )
{
    if( me != NULL )
    {
        me->incrementNum_ = theValue;
    }
}


/*+*****************************************************************************
 * Input:     isEmptyFlag: If true, invoke the callback function when the memory
 *                         pool is empty.
 *
 * Output:    None.
 *
 * InOut:     me: The memory allocator whose callback function can be invoked
 *                when the memory pool is empty.
 *
 * Function:  Control over using of callback when out of memory.
 *
 ****************************************************************************-*/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
void RiCMemoryAllocator_callMemoryPoolIsEmpty( RiCMemoryAllocator *const me,
                                               const RiCBoolean isEmptyFlag )
{
    if( me != NULL )
    {
        me->memoryPoolIsEmptyFlag_ = isEmptyFlag;
    }
}


/*+*****************************************************************************
 * Input:     newAllocator: Function that performs memory pool allocation.
 *
 * Output:    None.
 *
 * InOut:     me: The memory allocator holding the allocation method.
 *
 * Function:  Set the allocation method.
 *
 ****************************************************************************-*/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
void RiCMemoryAllocator_setAllocator( RiCMemoryAllocator *const me,
                                      RhpAddress (*const newAllocator)(RhpPositive numOfInstances) )
{
    if( me != NULL )
    {
        me->allocator_ = newAllocator;
    }
}

void RiCMemoryAllocator_poolReallocationMsg(const RhpString className, RhpPositive incrementNumber)
{
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
	fprintf(stderr,																			
			"%s: Memory pool is empty, allocating memory pool for additional %s instances.\n",	
			className ,	ARC_int2String(incrementNumber));
#endif
}

#endif
