/*

	Licensed Materials - Property of IBM
	(c) Copyright IBM Corporation 2006, 2016. All Rights Reserved.
*/

/******************************************************************
 ---------------------------------------------------------------
|							  RiCMap.c                          |
 --------------------------------------------------------------- 
*******************************************************************/
#ifndef MXF_SOURCE
#define MXF_SOURCE
#endif
#ifndef MXF_RICMAP_SOURCE
#define MXF_RICMAP_SOURCE
#endif

/* Includes *******************************************************************/

#include "RiCMap.h"
#ifdef RIC_USE_RICMAP
/* Constants ******************************************************************/

/* Macros *********************************************************************/

/* Types **********************************************************************/

/* The color of each node in the tree. */
typedef enum
{
    BLACK,
    RED,
    RB_NODE_COLOR_LAST
} RBNodeColor; /*lint -esym(749,RB_NODE_COLOR_LAST) not referenced */

/* The structure of each node in the tree. */
typedef struct RBNodeTag
{
    struct RBNodeTag *parent; /* parent node */
    struct RBNodeTag *left;   /* left child node (smaller key) */
    struct RBNodeTag *right;  /* right child node (larger key) */
    RBNodeColor color;        /* node color */
    gen_ptr key;              /* node key */
    gen_ptr value;            /* pointer to user data value */
} RBNode;

/* Variables ******************************************************************/

/* Prototypes *****************************************************************/

STATIC RiCBoolean checkIfKeyGreater(const_gen_ptr k1, const_gen_ptr k2);
STATIC RBNode *getRBNode(const RiCMap * const me, const_gen_ptr pKey);
STATIC RBNode *getSmallestRBNode(const RiCMap * const me, RBNode *node);
STATIC RBNode *getSuccessorRBNode(const RiCMap * const me, RBNode *node);
STATIC RiCBoolean removeNode(RiCMap * const me, RBNode * const nodeToDelete);
STATIC void rebalanceAfterAdd(RiCMap *me, RBNode *nodeX);
STATIC void rebalanceAfterRemove(RiCMap *me, RBNode *nodeX);
STATIC void rotateRight(RiCMap * const me, RBNode *nodeX);
STATIC void rotateLeft(RiCMap * const me, RBNode *nodeY);

/* Functions ******************************************************************/

/*+*****************************************************************************
 * Input:     isG: The key compare function.
 *
 * Output:    RiCMap_Create(): The newly created map or NULL on failure.
 *
 * InOut:     None.
 *
 * Function:  Create the map.
 *
 *
 *
 *
 *
 ****************************************************************************-*/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
RiCMap *RiCMap_Create(const RiCMapKeyIsGreater isGreater)
{
	/* Justification: Violation of Misra98 Rule45 / Misra04 Rule11.4				*/
	/* Cast from void* pointer is required when using the memory allocation utility */
	/*LDRA_INSPECTED 94 S */
	/*LDRA_INSPECTED 95 S */ /*LDRA_INSPECTED 203 S */
    RiCMap *me = (RiCMap*)RiCGetMemory( sizeof( RiCMap ));

    if( me != NULL )
    {
        /* Attempt to initialize the newly created map.  If the initialization
           fails, we must destroy the newly created map and return NULL. */
        if(RiCMap_Init( me, isGreater ) == RiCFALSE)
        {
            RiCMap_Destroy(me);
            me = NULL;
        }
    }

	/*LDRA_INSPECTED 71 S */

#ifdef USE_ASSERT
	RiC_ASSERT(me != NULL);
#endif

    return me;
}


/*+*****************************************************************************
 * Input:     isG: The compare function.
 *
 * Output:    RiCMap_Init(): Returns true if successful.
 *
 * InOut:     me: The map to initialize.
 *
 * Function:  Initialize the map.
 *
 *
 *
 *
 *
 ****************************************************************************-*/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
RiCBoolean RiCMap_Init(RiCMap * const me, const RiCMapKeyIsGreater isGreater)
{
    RiCBoolean result = RiCFALSE;
    RBNode *curSentinel = NULL;

    if( me != NULL )
    {
        /* The map is initially empty. */
        me->count = 0;

        /* Save away the key comparison function - if NULL, use the default. */
        if( isGreater != NULL )
        {
            me->isG = isGreater;
        }
        else
        {
			/* Justification: Violation of Misra98 Rule45 / Misra04 Rule11.4				  */
			/* Cast to void* pointer is required to implement general purpose data structures */
			/*LDRA_INSPECTED 94 S */
			/*LDRA_INSPECTED 93 S */
            me->isG = (RiCMapKeyIsGreater) &checkIfKeyGreater;
        }
		
		/* Justification: Violation of Misra98 Rule45 / Misra04 Rule11.4				*/
		/* Cast from void* pointer is required when using the memory allocation utility */
        /* Attempt to create a shared, black sentinel node to be used for all
           leaf nodes in the tree.  For completeness, the sentinel's left and
           right children are linked to the sentinel as well.  The key and
           value are always NULL.  In some cases, the algorithm will modify
           the parent node of the sentinel to assist with rebalancing. */
			/*LDRA_INSPECTED 94 S */
			/*LDRA_INSPECTED 95 S */ /*LDRA_INSPECTED 203 S */
        curSentinel = (RBNode*) RiCGetMemory( sizeof( RBNode ));
        if( curSentinel != NULL )
        {
            curSentinel->parent = NULL;
            curSentinel->left = curSentinel;
            curSentinel->right = curSentinel;
            curSentinel->color = BLACK;
            curSentinel->key = NULL;
            curSentinel->value = NULL;

            /* Save the shared sentinel in our map.  The root is initially
               set to the sentinel when the tree is empty. */
            me->sentinel = curSentinel;
            me->root = me->sentinel;

            /* We successfully initialized. */
            result = RiCTRUE;
        }

        /* If we had a memory allocation failure creating the sentinel, set
           our attributes to NULL and return false. */
        else
        {
            me->sentinel = NULL;
            me->root = NULL;
        }
    }

    return result;
}


/*+*****************************************************************************
 * Input:     None.
 *
 * Output:    None.
 *
 * InOut:     me: The map to destroy.
 *
 * Function:  Destroy the map.
 *
 *
 *
 *
 *
 ****************************************************************************-*/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
void RiCMap_Destroy(RiCMap * const me)
{
    if( me != NULL )
    {
        RiCMap_Cleanup(me);
        RiCReturnMemory(me);
    }
}


/*+*****************************************************************************
 * Input:     None.
 *
 * Output:    None.
 *
 * InOut:     me: The map to cleanup.
 *
 * Function:  Cleanup the map.
 *
 *
 *
 *
 *
 ****************************************************************************-*/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
void RiCMap_Cleanup(RiCMap * const me)
{
    if( me != NULL )
    {
        /* Remove all map nodes. */
        RiCMap_removeAll(me);

        /* Clear out our attributes */
        me->isG = NULL;
        me->root = NULL;

        /* Delete our shared sentinel if it was successfully created. */
        if( me->sentinel != NULL )
        {
            RiCReturnMemory(me->sentinel);
            me->sentinel = NULL;
        }
    }
}


/*+*****************************************************************************
 * Input:     me: The map to query.
 *
 * Output:    RiCMap_isEmpty(): Returns true if the map is empty.
 *
 * InOut:     None.
 *
 * Function:  Check if the map is empty.
 *
 *
 *
 *
 *
 ****************************************************************************-*/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
RiCBoolean RiCMap_isEmpty(const RiCMap * const me)
{
    RiCBoolean retval = RiCTRUE;

    if( me != NULL )
    {
        if( me->count != 0 ){
			retval = RiCFALSE;
		}
    }

    return retval;
}


/*+*****************************************************************************
 * Input:     me: The map to query.
 *
 * Output:    RiCMap_getCount(): Returns the number of elements in the map.
 *
 * InOut:     None.
 *
 * Function:  Get the number of elements in the map.
 *
 *
 *
 *
 *
 ****************************************************************************-*/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
RiC_INT32 RiCMap_getCount(const RiCMap * const me)
{
    RiC_INT32 result = 0;

    if( me != NULL )
    {
        result = me->count;
    }

    return result;
}


/*+*****************************************************************************
 * Input:     key: The unique key for the element.
 *
 *            value: The generic pointer element to add to the map.
 *
 * Output:    RiCMap_add(): Returns true if successful or false if there is an
 *                          attempt to insert a duplicate key or the RBNode
 *                          memory allocation fails.
 *
 * InOut:     me: The map in which to add the element.
 *
 * Function:  Add an element to the map.
 *
 *
 *
 *
 *
 ****************************************************************************-*/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
RiCBoolean RiCMap_add(RiCMap * const me, gen_ptr const pKey, gen_ptr const pValue)
{
    RiCBoolean result = RiCFALSE;

    if( me != NULL )
    {
        RBNode *current = me->root;
        RBNode *pParent = NULL;

        /* First find the parent that will own this new node.  If we
           discover this is a duplicate key along the way, we will return
           false. */
        result = RiCTRUE;
        while( (result != RiCFALSE) && (current != NULL) && (current != (me->sentinel)) )
        {
            /* Check for an attempt to insert a duplicate key. */
			/*LDRA_INSPECTED 45 D : LDRA Analysis incorrect. Pointer is checked for null before use */
            if( pKey == (current->key) ) 
            {
                result = RiCFALSE;
#ifdef USE_ASSERT
				RiC_ASSERT(result != RiCFALSE);
#endif
            }

            /* Otherwise, traverse the approprite child based upon the key
               comparison - left child if key is smaller, right child if
               key is greater.  All new nodes are inserted at the bottom
               of the tree. */
            else
            {
                pParent = current;
				/*LDRA_INSPECTED 19 D : LDRA Analysis incorrect. Function Pointer call has no available function definition  */
				/*LDRA_INSPECTED 41 D : LDRA Analysis incorrect. Function Pointer call has no available function declaration */	/*LDRA_INSPECTED 496 S */
                if( me->isG( pKey, current->key ) != RiCFALSE )
                {
                    current = current->right;
                }
                else
                {
                    current = current->left;
                }
            }
        }

        /* So long as we have not found a duplicate key, let's attempt to
           insert the new node into the tree. */
        if(result != RiCFALSE)
        {
			/* Justification: Violation of Misra98 Rule45 / Misra04 Rule11.4				*/
			/* Cast from void* pointer is required when using the memory allocation utility */
            /* Attempt to allocate a new RBNode. */
			/*LDRA_INSPECTED 94 S */
			/*LDRA_INSPECTED 95 S */ /*LDRA_INSPECTED 203 S */
            RBNode *newNode = (RBNode*) RiCGetMemory( sizeof( RBNode ));
            if( newNode != NULL )
            {
                /* Each new node is colored red and has two black sentinel
                   leaf nodes as chidren. */
                newNode->parent = pParent;
                newNode->left = me->sentinel;
                newNode->right = me->sentinel;
                newNode->color = RED;
                newNode->key = pKey;
                newNode->value = pValue;

                /* If we found a parent to insert into, add the new node as
                   the left or right child based upon its key.  Otherwise,
                   the new node is the root of our tree. */
                if( pParent != NULL )
                {
					/*LDRA_INSPECTED 19 D : LDRA Analysis incorrect. Function Pointer call has no available function definition  */
					/*LDRA_INSPECTED 41 D : LDRA Analysis incorrect. Function Pointer call has no available function declaration */	/*LDRA_INSPECTED 496 S */
                    if(me->isG( pKey, pParent->key ) != RiCFALSE)
                    {
                        pParent->right = newNode;
                    }
                    else
                    {
                        pParent->left = newNode;
                    }
                }
                else
                {
                    me->root = newNode;
                }

                /* Rebalance the tree after the add. */
                rebalanceAfterAdd(me, newNode);

                /* Bump up the count of nodes in the map. */
                me->count++;
            }

            /* Otherwise, we had a memory allocation error.  In this case,
               we just return false. */
            else
            {
                result = RiCFALSE;
            }
        }
    }

#ifdef USE_ASSERT
	RiC_ASSERT(result != RiCFALSE);
#endif

    return result;
}


/*+*****************************************************************************
 * Input:     key: The unique key for the map node to remove.
 *
 * Output:    RiCMap_removeKey(): Returns true if successful.  Otherwise, false.
 *
 * InOut:     me: The map from which to remove the map node.
 *
 * Function:  Remove a map node from the map using its key.
 *
 *
 *
 *
 *
 ****************************************************************************-*/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
RiCBoolean RiCMap_removeKey(RiCMap * const me, const_gen_ptr pKey )
{
    RiCBoolean result = RiCFALSE;
    RBNode *nodeToDelete;

    if( me != NULL )
    {
        /* First, find the node to delete. */
        nodeToDelete = getRBNode(me, pKey);

        /* If we found it, delete it and return true. */
        if( nodeToDelete != NULL )
        {
            result = removeNode(me, nodeToDelete);
        }
    }

    return result;
}


/*+*****************************************************************************
 * Input:     value: The generic pointer element to remove from the map.
 *
 * Output:    RiCMap_remove(): Returns true if successful.  Otherwise, false.
 *
 * InOut:     me: The map from which to remove the element.
 *
 * Function:  Remove an element from the map.
 *
 *
 *
 *
 *
 ****************************************************************************-*/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
RiCBoolean RiCMap_remove(RiCMap * const me, const const_gen_ptr pValue)
{
    RiCBoolean result = RiCFALSE;

    if( me != NULL )
    {
        RiCBoolean found = RiCFALSE;

        /* Start the search with the smallest node. */
        RBNode *node = getSmallestRBNode(me, me->root);
        if( node != NULL )
        {
            while( (found == RiCFALSE) && (node != NULL) && (node != (me->sentinel)) )
            {
                /* If we found the node, remove it. */
				/*LDRA_INSPECTED 45 D : LDRA Analysis incorrect. Pointer is checked for null before use */
                if( (node->value) == pValue )
                {
                    found = RiCTRUE;
                    result = removeNode(me, node);
                }
                /* Otherwise, check the next successor node. */
                else
                {
                    node = getSuccessorRBNode(me, node);
                }
            }
        }
    }

#ifdef USE_ASSERT
	RiC_ASSERT(result != RiCFALSE);
#endif

    return result;
}


/*+*****************************************************************************
 * Input:     None.
 *
 * Output:    None.
 *
 * InOut:     me: The map from which all elements are removed.
 *
 * Function:  Remove all elements from the map.
 *
 *
 *
 *
 *
 ****************************************************************************-*/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
void RiCMap_removeAll( RiCMap * const me )
{
    if( me != NULL )
    {
        while( (me->root) != (me->sentinel) )
        {
            (void)removeNode(me, me->root);
        }
    }
}


/*+*****************************************************************************
 * Input:     me: The map to access.
 *
 *            key: The unique key for the element to retrieve.
 *
 * Output:    RiCMap_getKey(): The element matching the given key or NULL if
 *                             not found.
 *
 * InOut:     None.
 *
 * Function:  Get element by its key.
 *
 *
 *
 *
 *
 ****************************************************************************-*/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
gen_ptr RiCMap_getKey(const RiCMap * const me, const_gen_ptr pKey)
{
    gen_ptr result = NULL;

    if( me != NULL )
    {
        /* Get the node matching the key and return its value. */
        RBNode *node = getRBNode(me, pKey);
        if( node != NULL )
        {
            result = node->value;
        }
    }

	/*LDRA_INSPECTED 71 S */
    return result;
}


/*+*****************************************************************************
 * Input:     me: The map to access.
 *
 *            index: The position of the element to return.
 *
 * Output:    RiCMap_getAt(): Returns the element at the given index or NULL if
 *                            the index is out of bounds.
 *
 * InOut:     None.
 *
 * Function:  Return the element at a given index.
 *
 *
 *
 *
 *
 ****************************************************************************-*/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
gen_ptr RiCMap_getAt(const RiCMap * const me, const RiC_INT32 index)
{
    gen_ptr result = NULL;
    RiC_INT32 theCount;

    if( me != NULL )
    {
        /* Start with the smallest node. */
        RBNode *node = getSmallestRBNode(me, me->root);
        if( node != NULL )
        {
            /* Get successor nodes until we've counted to the index. */
            for( theCount = 0; ( (theCount < index) && (node != (me->sentinel)) );
                 theCount++ )
            {
                node = getSuccessorRBNode(me, node);
            }
#ifdef USE_ASSERT
			if (node == me->sentinel) /* ended before getting to the index */
				RiC_ASSERT(NULL);
#endif
            /* If we found a node, return it's value.  Note that if we
               end out finding a sentinel, it's value will be NULL. */
            if( node != NULL )
            {
                result = node->value;
            }
        }
    }

	/*LDRA_INSPECTED 71 S */
    return result;
}


/*+*****************************************************************************
 * Input:     me: The map to query.
 *
 *            key: The unique key for the map element to look up.
 *
 * Output:    value: The element matching the given key or NULL if not found.
 *
 *            RiCMap_lookUp(): Returns true if found.
 *
 * InOut:     None.
 *
 * Function:  Find an element in the map by its key.
 *
 *
 *
 *
 *
 ****************************************************************************-*/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
RiCBoolean RiCMap_lookUp(const RiCMap * const me, const_gen_ptr pKey, gen_ptr *pValue)
{
    RiCBoolean result = RiCFALSE;

    if( (me != NULL) && (pValue != NULL) )
    {
        /* Get the node matching the key. */
        RBNode *node = getRBNode(me, pKey);
        if( node != NULL )
        {
            /* Save away its value as an OUT param and return true. */
            *pValue = node->value;
            result = RiCTRUE;
        }
        else
        {
            *pValue = NULL;
        }
    }

    return result;
}


/*+*****************************************************************************
 * Input:     me: The map to search.
 *
 *            value: The map value to search for.
 *
 * Output:    RiCMap_find(): Returns true if found.
 *
 * InOut:     None.
 *
 * Function:  Find a map value in the map.
 *
 *
 *
 *
 *
 ****************************************************************************-*/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
RiCBoolean RiCMap_find(const RiCMap * const me, const const_gen_ptr pValue)
{
    RiCBoolean result = RiCFALSE;

    if( me != NULL )
    {
        /* Start with the smallest node. */
        RBNode *node = getSmallestRBNode(me, me->root);
        if( node != NULL )
        {
            while( (result == RiCFALSE) && (node != NULL) && (node != (me->sentinel)) )
            {
                /* If we found the node, return true. */
				/*LDRA_INSPECTED 45 D : LDRA Analysis incorrect. Pointer is checked for null before use */
                if( (node->value) == pValue )
                {
                    result = RiCTRUE;
                }
                /* Otherwise, get the next successor node. */
                else
                {
                    node = getSuccessorRBNode(me, node);
                }
            }
        }
    }

    return result;
}


/*+*****************************************************************************
 * Input:     me: The map on which to iterate.
 *
 * Output:    pos: The iterator - set to the first element in the map.
 *
 *            RiCMap_first(): The first element in the map or NULL if the
 *                            map is empty.
 *
 * InOut:     None.
 *
 * Function:  Move to the first map value.
 *
 *
 *
 *
 *
 ****************************************************************************-*/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
gen_ptr RiCMap_first(const RiCMap * const me, gen_ptr *pPos)
{
    gen_ptr result = NULL;

    if( (me != NULL) && (pPos != NULL) )
    {
        /* Start with the smallest node. */
        RBNode *first = getSmallestRBNode(me, me->root);

        /* Setup the iterator to point at the node and return its value. */
        if( first != NULL)
        {
			/* Justification: Violation of Misra98 Rule45 / Misra04 Rule11.4						*/
			/* Cast from/to void* pointer is required to implement general purpose data collections */
			/*LDRA_INSPECTED 93 S */
			/*LDRA_INSPECTED 95 S */ /*LDRA_INSPECTED 203 S */
            *pPos = (gen_ptr) first;
            result = first->value;
        }
        else
        {
#ifdef USE_ASSERT
			RiC_ASSERT(NULL);
#endif
			/* Justification: Violation of Misra98 Rule45 / Misra04 Rule11.4						*/
			/* Cast from/to void* pointer is required to implement general purpose data collections */
			/*LDRA_INSPECTED 93 S */
			/*LDRA_INSPECTED 95 S */ /*LDRA_INSPECTED 203 S */
            *pPos = (gen_ptr) me->sentinel;
        }
    }
#ifdef USE_ASSERT
	else /* map or destination are NULL */
	{
		RiC_ASSERT(NULL);
	}
#endif

	/*LDRA_INSPECTED 71 S */
    return result;
}


/*+*****************************************************************************
 * Input:     me: The map on which to iterate.
 *
 * Output:    RiCMap_next(): The next element in the map or NULL if reached
 *                           the end of the map.
 *
 * InOut:     pos: The iterator - incremented to the next element in the map.
 *
 * Function:  Return the next map value.
 *
 *
 *
 *
 *
 ****************************************************************************-*/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
gen_ptr RiCMap_next(const RiCMap * const me, gen_ptr *pPos)
{
    gen_ptr result = NULL;

    if( (me != NULL) && (pPos != NULL) )
    {
		/* Justification: Violation of Misra98 Rule45 / Misra04 Rule11.4						*/
		/* Cast from/to void* pointer is required to implement general purpose data collections */
        /* Get the next successor node. */
		/*LDRA_INSPECTED 95 S */ /*LDRA_INSPECTED 203 S */
		/*LDRA_INSPECTED 98 S */ /*LDRA_INSPECTED 458 S */
        RBNode *pNext = getSuccessorRBNode(me, (RBNode*)(*pPos));

        /* Setup the iterator to point at the node and return its value. */
        if( pNext != NULL)
        {
			/* Justification: Violation of Misra98 Rule45 / Misra04 Rule11.4						*/
			/* Cast from/to void* pointer is required to implement general purpose data collections */
			/*LDRA_INSPECTED 93 S */
			/*LDRA_INSPECTED 95 S */ /*LDRA_INSPECTED 203 S */
             *pPos = (gen_ptr) pNext;
            result = pNext->value;
        }
        else
        {
			/* Justification: Violation of Misra98 Rule45 / Misra04 Rule11.4						*/
			/* Cast from/to void* pointer is required to implement general purpose data collections */
 			/*LDRA_INSPECTED 93 S */
			/*LDRA_INSPECTED 95 S */ /*LDRA_INSPECTED 203 S */
			*pPos = (gen_ptr) me->sentinel;
        }
    }

	/*LDRA_INSPECTED 71 S */
    return result;
}


/*+*****************************************************************************
 * Input:     me: The map on which to iterate.
 *
 *            pos: The iterator.
 *
 * Output:    RiCMap_get(): Returns the element at the given iterator
 *                          position or NULL if out of bounds.
 *
 * InOut:     None.
 *
 * Function:  Return the map value at the given pos.
 *
 *
 *
 *
 *
 ****************************************************************************-*/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
gen_ptr RiCMap_get(const RiCMap * const me, const const_gen_ptr pPos )
{
    gen_ptr result = NULL;

    if( (me != NULL) && (pPos != NULL) )
    {
		/* Justification: Violation of Misra98 Rule45 / Misra04 Rule11.4						*/
		/* Cast from/to void* pointer is required to implement general purpose data collections */
		/*LDRA_INSPECTED 95 S */ /*LDRA_INSPECTED 203 S */
        result = ((RBNode*)pPos)->value;
    }

	/*LDRA_INSPECTED 71 S */
    return result;
}


/*+*****************************************************************************
 * Input:     me: The map on which to iterate.
 *
 *            pos: The iterator.
 *
 * Output:    RiCMap_isDone(): Returns true when iterator reaches the
 *                             last element in the map.
 *
 * InOut:     None.
 *
 * Function:  Check of last element in iteration.
 *
 *
 *
 *
 *
 ****************************************************************************-*/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
RiCBoolean RiCMap_isDone(const RiCMap * const me, const const_gen_ptr pPos )
{
    RiCBoolean result = RiCTRUE;

    if( (me != NULL) && (pPos != NULL) )
    {
        /* We're done when the iterator reaches the sentinel. */
        result = (RiCBoolean) ( pPos == (me->sentinel) );
    }

    return result;
}


/*+*****************************************************************************
 * Input:     k1: The first key to compare.
 *
 *            k2: The second key to compare.
 *
 * Output:    checkIfKeyGreater(): Returns true if key 1 is greater than
 *                                 key 2.
 *
 * InOut:     None.
 *
 * Function:  Default compare function.
 *
 *
 *
 *
 *
 ****************************************************************************-*/
STATIC RiCBoolean checkIfKeyGreater(const_gen_ptr k1, const_gen_ptr k2)
{
	/* Justification: Violation of Misra98 Rule45 / Misra04 Rule11.4 */
	/* Cast is required to implement sorting of map elements		 */
	/*LDRA_INSPECTED 439 S */ /*LDRA_INSPECTED 203 S */
	/*LDRA_INSPECTED 439 S */ /*LDRA_INSPECTED 203 S */
	return ((RiCBoolean) (((Rhp_uintptr_t) k1) > ((Rhp_uintptr_t) k2)));
}


/*+*****************************************************************************
 * Input:     me: The map to access.
 *
 *            key: The unique key for RBNode to retrieve.
 *
 * Output:    getRBNode(): The RBNode matching the given key or NULL if
 *                         not found.
 *
 * InOut:     None.
 *
 * Function:  Get an RBNode by its key.
 *
 *
 *
 *
 *
 ****************************************************************************-*/
STATIC RBNode *getRBNode(const RiCMap * const me, const_gen_ptr pKey )
{
    RBNode *result = NULL;

    if( me != NULL )
    {
        RBNode *current = me->root;
        while( (result == NULL) && (current != NULL) && (current != (me->sentinel)) )
        {
            /* We found the matching key. */
			/*LDRA_INSPECTED 45 D : LDRA Analysis incorrect. Pointer is checked for null before use */
            if( pKey == (current->key) )
            {
                result = current;
            }

            /* Otherwise, search left or right based upon the key comparison. */
            else
            {
				/*LDRA_INSPECTED 19 D : LDRA Analysis incorrect. Function Pointer call has no available function definition  */
				/*LDRA_INSPECTED 41 D : LDRA Analysis incorrect. Function Pointer call has no available function declaration */	/*LDRA_INSPECTED 496 S */
                if(me->isG(pKey, current->key) != RiCFALSE)
                {
                    current = current->right;
                }
                else
                {
                    current = current->left;
                }
            }
        }
    }

	/*LDRA_INSPECTED 71 S */
    return result;
}


/*+*****************************************************************************
 * Input:     me: The map to search.
 *
 *            node: The RBNode to start from.
 *
 * Output:    getSmallestRBNode(): The smallest RBNode under the given node.
 *
 * InOut:     None.
 *
 * Function:  Returns the smallest RBNode under the given node.
 *
 *
 *
 *
 *
 ****************************************************************************-*/
STATIC RBNode *getSmallestRBNode(const RiCMap * const me, RBNode *node)
{
    if( (me != NULL) && ((node != NULL) && (node != (me->sentinel))) )
    {
        /* Return the farthest left node. */
		/*LDRA_INSPECTED 45 D : LDRA Analysis incorrect. Pointer is checked for null before use */
        while( (node->left) != (me->sentinel) )
        {
            node = node->left;
        }
    }

    return node;
}

/*+*****************************************************************************
 * Input:     me: The map to search.
 *
 *            node: The RBNode to start from.
 *
 * Output:    getSuccessorRBNode(): The next successor for the given RBNode.
 *
 * InOut:     None.
 *
 * Function:  Returns the next successor for the given RBNode.
 *
 *
 *
 *
 *
 ****************************************************************************-*/
STATIC RBNode *getSuccessorRBNode(const RiCMap * const me, RBNode *node)
{
    if( (me != NULL) && (node != NULL) && (node != (me->sentinel)) )
    {
        /* If a right subtree exists, return the left most node of the right
           subtree of the given node. */
        if( (node->right) != (me->sentinel) )
        {
            node = getSmallestRBNode(me, node->right);
        }

        /* Otherwise, travel up from the node until we find a node left of
           its parent node. */
        else
        {
            RBNode *pParent = node->parent;
            while( (pParent != NULL) &&
                   ((pParent != (me->sentinel)) &&
                    (node == (pParent->right))) )
            {
				/*LDRA_INSPECTED 45 D : LDRA Analysis incorrect. Pointer is checked for null before use */
                node = pParent;
                pParent = node->parent;
            }

            if( pParent != NULL )
            {
                node = pParent;
            }
            else
            {
                node = me->sentinel;
            }
        }
    }

    return node;
}


/*+*****************************************************************************
 * Input:     None.
 *
 * Output:    removeNode(): Returns true if successful.  Otherwise, false.
 *
 * InOut:     me: The map from which to remove the node.
 *
 *            nodeToDelete: The node to delete from the map.
 *
 * Function:  Remove a node from the map.  Return true if successful.
 *
 *
 *
 *
 *
 ****************************************************************************-*/
STATIC RiCBoolean removeNode(RiCMap * const me, RBNode * const nodeToDelete )
{
	RiCBoolean result = RiCFALSE;
	RBNode *successor;
	RBNode *successorChild;
	
	if( (me != NULL) && (nodeToDelete != NULL) )
	{
		/* If the node to delete has a sentinel as a child, then the
	   successor is that node. */
		if( ((nodeToDelete->left) == (me->sentinel)) ||	((nodeToDelete->right) == (me->sentinel)) )
		{
			successor = nodeToDelete;
		}
		
		/* Otherwise, find a successor with a sentinel as a child. */
		else
		{
			successor = nodeToDelete->right;
			if (successor != NULL)
			{
				while( (successor->left) != (me->sentinel) )
				{
					successor = successor->left;
				}
			}
		}

		/* Get the child of the tree successor.  Note that in some cases, the
           child will be a sentinel. */
		if (successor != NULL)
		{
			if( (successor->left) != (me->sentinel) )
			{
				successorChild = successor->left;
			}
			else
			{
				successorChild = successor->right;
			}

			/* Now remove the successor from the tree by linking its
			   child to its parent and visa-versa.  If the parent is NULL,
			   the child becomes the tree root. */
			if( successorChild != NULL)
			{
				successorChild->parent = successor->parent;
				if( (successor->parent) != NULL )
				{
					if( successor == (successor->parent->left) )
					{
						successor->parent->left = successorChild;
					}
					else
					{
						successor->parent->right = successorChild;
					}
				}
				else
				{
					me->root = successorChild;
				}

				/* If the successor is not the node to delete, then we must swap
				   the successor's key and value into the node to delete in
				   preparation of deleting the successor. */
				if( successor != nodeToDelete )
				{
					nodeToDelete->key = successor->key;
					nodeToDelete->value = successor->value;
				}

				/* We need to rebalance the tree if the successor was black, since
				   deleting a black node changes the black height. */
				if( successor->color == BLACK )
				{
					rebalanceAfterRemove(me, successorChild);
				}

				/* Finally, delete the successor. */
				RiCReturnMemory(successor);

				/* Decrement the count of nodes in the map. */
				me->count--;

				/* We were successful. */
				result = RiCTRUE;
			}
		}
	}
	return result;
}


/*+*****************************************************************************
 * Input:     None.
 *
 * Output:    None.
 *
 * InOut:     me: The map to rebalance.
 *
 *            nodeX:  The newly added node.
 *
 * Function:  Maintains the Red-Black tree balance after adding nodeX.
 *
 *
 *
 *
 *
 ****************************************************************************-*/
STATIC void rebalanceAfterAdd(RiCMap *me, RBNode *nodeX)
{
    /* In the RiCMap_add function, we always add a red node with two black
       sentinel children to a node on the bottom of the tree.  If the node's
       parent is black, we're all set.  However, if the parent is red, we've
       got two reds in a row which must be fixed.  Note that since the root
       is always black, if our parent is red, we are guarenteed to have a
       grandparent.
       
       If our uncle is red, reversing our parents color will either solve
       the problem or push it higher up the tree.  Note that after
       recoloring our parent, the black height of the tree and the shape
       of the tree are unchanged and we are done if our great-grandparent
       is black. */

	if ((me != NULL) && (nodeX != NULL) && ((nodeX->parent) != NULL))
	{
		while( (nodeX != (me->root)) && ((nodeX->parent->color) == RED) )
		{
			/* x's parent is a left child. */
			RBNode *nodeXParent = nodeX->parent;
			RBNode *uncle = NULL;
			if ((nodeXParent->parent) != NULL) 
			{
				if(nodeXParent == (nodeXParent->parent->left))
				{
					/* Get x's uncle. */
					uncle = nodeXParent->parent->right;

					/* x's uncle is red.  Change x's parent and uncle to black and
					   change x's grandparent to red.  Repeat this process with
					   x's grandparent. */
					if (uncle != NULL)
					{
						if( uncle->color == RED )
						{
							nodeXParent->color = BLACK;
							uncle->color = BLACK;
							nodeXParent->parent->color = RED;
							nodeX = nodeX->parent->parent;
						}

						/* x's uncle is black */
						else
						{
							/* If x is a right child, we rotate it so it switches levels
							   with its parent.  We then operate on the old parent just
							   like if x was a left child. */
							if( nodeX == (nodeX->parent->right) )
							{
								nodeX = nodeX->parent;
								rotateLeft(me, nodeX);
							}

							/* When x is left child (or we just rotated the old parent
							   into x's old position above), we need to recolor and
							   rotate. */
							nodeX->parent->color = BLACK;
							nodeXParent = nodeX->parent;
							nodeXParent->parent->color = RED;
							rotateRight(me, nodeX->parent->parent);
						}
					}
				}

				/* x's parent is a right child.  This is a mirror image of the code
				   above, the parent and uncle positions are swapped. */
				else
				{
					/* Get x's uncle. */
					uncle = nodeXParent->parent->left;

					/* x's uncle is red.  Change x's parent and uncle to black and
					   change x's grandparent to red.  Repeat this process with
					   x's grandparent. */
					if (uncle != NULL)
					{
						if( uncle->color == RED )
						{
							nodeX->parent->color = BLACK;
							uncle->color = BLACK;
							nodeXParent->parent->color = RED;
							nodeX = nodeX->parent->parent;
						}

						/* x's uncle is black */
						else
						{
							/* If x is a left child, we rotate it so it switches levels
							   with its parent.  We then operate on the old parent just
							   like if x was a right child. */
							if( nodeX == (nodeX->parent->left) )
							{
								nodeX = nodeX->parent;
								rotateRight(me, nodeX);
							}

							/* When x is right child (or we just rotated the old parent
							   into x's old position above), we need to recolor and
							   rotate. */
							nodeXParent = nodeX->parent;
							nodeXParent->color = BLACK;
							nodeXParent->parent->color = RED;
							rotateLeft(me, nodeX->parent->parent);
						}
					}
				}
			}
		}

	}
	if ((me != NULL) && (me->root != NULL))
	{
		/* The root node is always black. */
		me->root->color = BLACK;
	}
}


/*+*****************************************************************************
 * Input:     None.
 *
 * Output:    None.
 *
 * InOut:     me: The map to rebalance.
 *
 *            nodeX:  The newly removed node,
 *
 * Function:  Maintains the Red-Black tree balance after removing nodeX.
 *
 *
 *
 *
 *
 ****************************************************************************-*/
STATIC void rebalanceAfterRemove(RiCMap *me, RBNode *nodeX)
{
	if ((me != NULL) && (nodeX != NULL) && ((nodeX->parent) != NULL))
	{
		while( (nodeX != (me->root)) && ((nodeX->color) == BLACK) )
		{
			RBNode *sibling = NULL;
			/* x's is the left sibling. */
			if( nodeX == (nodeX->parent->left) )
			{
				/* Get our other sibling. */
				sibling = nodeX->parent->right;

				if (sibling != NULL)
				{
					/* Red sibling */
					if( sibling->color == RED )
					{
						sibling->color = BLACK;
						nodeX->parent->color = RED;
						rotateLeft(me, nodeX->parent);
						sibling = nodeX->parent->right;
					}

					if ( ((sibling->left) != NULL) && ((sibling->right) != NULL) )
					{
						/* Sibling with black nephews */
						if( ((sibling->left->color) == BLACK) &&
							((sibling->right->color) == BLACK) )
						{
							sibling->color = RED;
							nodeX = nodeX->parent;
						}

						/* Sibling with mixed nephews or red nephews */
						else
						{
							if( sibling->right->color == BLACK )
							{
								sibling->left->color = BLACK;
								sibling->color = RED;
								rotateRight(me, sibling);
								sibling = nodeX->parent->right;
							}
							sibling->color = nodeX->parent->color;
							nodeX->parent->color = BLACK;
							sibling->right->color = BLACK;
							rotateLeft(me, nodeX->parent);
							nodeX = me->root;
						}
					}
				}
			}
        
			/* x's is the right sibling.  Reverse the algorithm above. */
			else
			{
				/* Get our other sibling. */
				sibling = nodeX->parent->left;
				if (sibling != NULL)
				{
					/* Red sibling */
					if( sibling->color == RED )
					{
						sibling->color = BLACK;
						nodeX->parent->color = RED;
						rotateRight(me, nodeX->parent);
						sibling = nodeX->parent->left;
					}

					if ( ((sibling->right) != NULL) && ((sibling->left) != NULL) )
					{
						/* Sibling with black nephews */
						if( ((sibling->right->color) == BLACK) &&
							((sibling->left->color) == BLACK) )
						{
							sibling->color = RED;
							nodeX = nodeX->parent;
						}

						/* Sibling with mixed nephews or red nephews */
						else
						{
							if( sibling->left->color == BLACK )
							{
								sibling->right->color = BLACK;
								sibling->color = RED;
								rotateLeft(me, sibling);
								sibling = nodeX->parent->left;
							}
							sibling->color = nodeX->parent->color;
							nodeX->parent->color = BLACK;
							sibling->left->color = BLACK;
							rotateRight(me, nodeX->parent);
							nodeX = me->root;
						}
					}
				}
			}
		}
	}
	if(nodeX != NULL){
		nodeX->color = BLACK;
	}
}


/*+*****************************************************************************
 * Input:     None.
 *
 * Output:    None.
 *
 * InOut:     me: The map to query.
 *
 *            nodeX:  The map node to rotate right.  See the diagram below for
 *                    more details on this process.
 *
 * Function:  Rotates an node in the tree to the right.  When rotating x to the
 *            right, the right child of x's left child (b) becomes x's left
 *            child and x takes that old position.
 *
 *                     x     ==>     y
 *                    / \           / \
 *                   y   z         a   x
 *                  / \               / \
 *                 a   b             b   z
 *
 *
 ****************************************************************************-*/
STATIC void rotateRight(RiCMap * const me, RBNode *nodeX)
{
    /* To assist in understanding this code, the variable names below match
       the diagram in the function description. */

	if ((me != NULL) && (nodeX != NULL))
	{
		/* Get x's left child (y). */
		RBNode *nodeY = nodeX->left;

		if (nodeY != NULL)
		{
			/* Replace x's left child with y's right child (b). */
			nodeX->left = nodeY->right;

			/* If b is not a sentinel, it's parent switches from y to x. */
			if( ((nodeY->right) != NULL) && ((nodeY->right) != (me->sentinel)) )
			{
				nodeY->right->parent = nodeX;
			}

			/* If y is not a sentinel, its parent becomes x's old parent. */
			if( nodeY != (me->sentinel) )
			{
				nodeY->parent = nodeX->parent;
			}

			/* If x has a parent, replace the child holding x with y. */
			if(nodeX->parent != NULL)
			{
				if( nodeX == (nodeX->parent->right) )
				{
					nodeX->parent->right = nodeY;
				}
				else
				{
					nodeX->parent->left = nodeY;
				}
			}

			/* Otherwise, x has no parent and y becomes the root of the tree. */
			else
			{
				me->root = nodeY;
			}

			/* Replace y's right child with x */
			nodeY->right = nodeX;

			/* Finally, if x is not a sentinel, its parent becomes y. */
			if( nodeX != (me->sentinel) )
			{
				nodeX->parent = nodeY;
			}
		}
	}
}


/*+*****************************************************************************
 * Input:     None.
 *
 * Output:    None.
 *
 * InOut:     me: The map to query.
 *
 *            nodeY:  The map node to rotate left.  See the diagram below for
 *                    more details on this process.
 *
 * Function:  Rotates an node in the tree to the left.  When rotating y to the
 *            left, the left child of y's right child (b) becomes y's right
 *            child and y takes that old position.
 *
 *                     x     <==     y
 *                    / \           / \
 *                   y   z         a   x
 *                  / \               / \
 *                 a   b             b   z
 *
 *
 *
 ****************************************************************************-*/
STATIC void rotateLeft(RiCMap * const me, RBNode *nodeY)
{
    /* To assist in understanding this code, the variable names below match
       the diagram in the function description. */

	if ((me != NULL) && (nodeY != NULL))
	{
		/* Get y's right child (x). */
		RBNode *nodeX = nodeY->right;

		if (nodeX != NULL)
		{
			/* Replace y's right child with x's left child (b). */
			nodeY->right = nodeX->left;
    
			/* If b is not a sentinel, it's parent switches from x to y. */
			if( ((nodeX->left) != NULL) && ((nodeX->left) != (me->sentinel)) )
			{
				nodeX->left->parent = nodeY;
			}

			/* If x is not a sentinel, its parent becomes y's old parent. */
			if( nodeX != (me->sentinel) )
			{
				nodeX->parent = nodeY->parent;
			}

			/* If y has a parent, replace that parent's child that holds y with x. */
			if( (nodeY->parent) != NULL )
			{
				if( nodeY == (nodeY->parent->left) )
				{
					nodeY->parent->left = nodeX;
				}
				else
				{
					nodeY->parent->right = nodeX;
				}
			}

			/* Otherwise, y has no parent and x becomes the root of the tree. */
			else
			{
				me->root = nodeX;
			}

			/* Replace x's left child with y */
			nodeX->left = nodeY;

			/* Finally, if y is not a sentinel, its parent becomes x. */
			if( nodeY != (me->sentinel) )
			{
				nodeY->parent = nodeX;
			}
		}
	}
}

#endif
