/********************************************************************************

 Purpose:    A map container implemented as a simple, iterative, Red-Black
              Tree.  This container provides worst case search, insert, and
              delete performance of O(lg n).  This map container supports NULL
              key and data values.

              Each node in the tree is colored either red or black.  The root
              node is always black.  All leaf nodes (sentinels) are also
              black.  This alogoritm uses a shared sentinel for all leaves.
              If a given node is colored red, then both of its child nodes
              must be colored black.  Every path from a given node to any of
              its leaves must contain exactly the same number of black nodes.


	Licensed Materials - Property of IBM
	(c) Copyright IBM Corporation 1999, 2011. All Rights Reserved.


 ---------------------------------------------------------------
|							  RiCMap.h                          |
 --------------------------------------------------------------- 
*********************************************************************************/

#ifndef RiCMap_H
#define RiCMap_H

/* Includes *******************************************************************/

#include "mxf_cfg.h"

/* Constants ******************************************************************/

/*LDRA_INSPECTED 141 S : Type forward declaration */
struct RBNodeTag;

/* Macros *********************************************************************/

/* Types **********************************************************************/

/* User supplied function, return true if k1 > k2 */
typedef RiCBoolean (*RiCMapKeyIsGreater) (const_gen_ptr k1, const_gen_ptr k2);

/* Keep misspelling for backward compatibility */
typedef RiCMapKeyIsGreater RiCMapKeyIsGrater;

/* The structure of the map. */
typedef struct RiCMapTag
{
    /* The root of Red-Black tree */
    struct RBNodeTag *root;

    /* The shared black sentinel for all leaves in the tree */
    struct RBNodeTag *sentinel;

    /* The count of nodes in the tree. */
    RiC_INT32 count;

    /* The key comparison function */
    RiCMapKeyIsGreater isG;
} RiCMap;

/* Prototypes *****************************************************************/

/* Creation and Destruction */
RiCMap *RiCMap_Create(const RiCMapKeyIsGreater isGreater);
RiCBoolean RiCMap_Init(RiCMap * const me, const RiCMapKeyIsGreater isGreater);
void RiCMap_Destroy(RiCMap * const me);
void RiCMap_Cleanup(RiCMap * const me);

/* Map methods */
RiCBoolean RiCMap_isEmpty(const RiCMap * const me);
RiC_INT32 RiCMap_getCount(const RiCMap * const me);
RiCBoolean RiCMap_add(RiCMap * const me, gen_ptr const pKey, gen_ptr const pValue);
RiCBoolean RiCMap_removeKey(RiCMap * const me, const_gen_ptr pKey);
RiCBoolean RiCMap_remove(RiCMap * const me, const const_gen_ptr pValue);
void RiCMap_removeAll(RiCMap * const me);
gen_ptr RiCMap_getKey(const RiCMap * const me, const_gen_ptr pKey);
gen_ptr RiCMap_getAt(const RiCMap * const me, const RiC_INT32 index);
RiCBoolean RiCMap_lookUp(const RiCMap * const me, const_gen_ptr pKey, gen_ptr *pValue);
RiCBoolean RiCMap_find(const RiCMap * const me, const const_gen_ptr pValue);

/* Iteration methods */
gen_ptr RiCMap_first(const RiCMap * const me, gen_ptr *pPos);
gen_ptr RiCMap_next(const RiCMap * const me, gen_ptr *pPos);
gen_ptr RiCMap_get(const RiCMap * const me, const const_gen_ptr pPos);
RiCBoolean RiCMap_isDone(const RiCMap * const me, const const_gen_ptr pPos);


#endif

