/*

	Licensed Materials - Property of IBM
	(c) Copyright IBM Corporation 2006, 2011. All Rights Reserved.
*/

/* Includes *******************************************************************/

#ifndef MXF_SOURCE
#define MXF_SOURCE
#endif
#ifndef MXF_RICMEMORYMANAGER_SOURCE
#define MXF_RICMEMORYMANAGER_SOURCE
#endif

#include <string.h>
#include "RiCMemoryManager.h"
#include "RiCOutput.h"

#ifdef RIC_USE_MEMORY_MANAGER

/* Constants ******************************************************************/

/*lint -save -e956 (Intentional use of non const, non volatile static to
  store the memory pools.  These variables are guarded with a mutex for
  multi-threaded access.) */

/* Create the memory pool structures. */
#if RIC_TINY_POOL_BLOCKS > 0U
STATIC RiCMemoryPool ricTinyMemoryPool;
#endif
#if RIC_SMALL_POOL_BLOCKS > 0U
static RiCMemoryPool ricSmallMemoryPool;
#endif
#if RIC_MEDIUM_POOL_BLOCKS > 0U
static RiCMemoryPool ricMediumMemoryPool;
#endif
#if RIC_TINY_POOL_BLOCKS > 0U
STATIC RiCMemoryPool ricLargeMemoryPool;
#endif
#if RIC_TINY_POOL_BLOCKS > 0U
static RiCMemoryPool ricHugeMemoryPool;
#endif

/* Create the memory block arrays for each memory pool */
#if RIC_TINY_POOL_BLOCKS > 0U
STATIC RiCTinyMemoryBlock ricTinyMemoryBlocks[RIC_TINY_POOL_BLOCKS];
#endif
#if RIC_SMALL_POOL_BLOCKS > 0U
STATIC RiCSmallMemoryBlock ricSmallMemoryBlocks[RIC_SMALL_POOL_BLOCKS];
#endif
#if RIC_MEDIUM_POOL_BLOCKS > 0U
STATIC RiCMediumMemoryBlock ricMediumMemoryBlocks[RIC_MEDIUM_POOL_BLOCKS];
#endif
#if RIC_LARGE_POOL_BLOCKS > 0U
STATIC RiCLargeMemoryBlock ricLargeMemoryBlocks[RIC_LARGE_POOL_BLOCKS];
#endif
#if RIC_HUGE_POOL_BLOCKS > 0U
STATIC RiCHugeMemoryBlock ricHugeMemoryBlocks[RIC_HUGE_POOL_BLOCKS];
#endif

/*lint -restore */

/* Macros *********************************************************************/

/* Types **********************************************************************/

/* Variables ******************************************************************/

/* Prototypes *****************************************************************/

STATIC RiCMemoryPool *getMemoryPool( const RiC_UINT32 invertedMemoryPoolId );

STATIC RiCMemoryBlockHeader *getMemoryBlockHeader( const RhpAddress pContents );

STATIC void initMemoryPool( RiCMemoryPool *const me,
                            const RiC_UINT32 memPoolId,
                            const RiC_UINT32 usableBSize,
                            const RiC_UINT32 totalBSize,
                            const RiC_UINT32 nBlocks,
                            RhpAddress startPool );

STATIC RiCBoolean checkIfMemoryPoolIsEmpty( const RiCMemoryPool *const me );

STATIC RhpAddress getMemoryFromMemoryPool( RiCMemoryPool *const me );

STATIC void returnMemoryToMemoryPool( RiCMemoryPool *const me,
                                      RhpAddress blockHeader );

STATIC RiCBoolean validateMemoryPool( const RiCMemoryPool *const me );

STATIC void getMemoryPoolUsage( RiCMemoryPool *const me,
                                RiCMemoryPoolUsage *usage );

/* Functions ******************************************************************/

/*+*****************************************************************************
 * Input:     None.
 *
 * Output:    None.
 *
 * InOut:     None.
 *
 * Function:  Initialize all of the pools within the memory manager.  This
 *            function must be invoked prior to allocating any memory.
 *
 *------------------------------------------------------------------------------
 * Author             Date        Modification
 * J.Collins,P.Urban  09.09.2005  Updates for MISRA-C and EADS coding standards
 *
 ****************************************************************************-*/
void RiCInitMemoryManager( void )
{
    /* Initialize the tiny memory pool. */
#if RIC_TINY_POOL_BLOCKS > 0U
    initMemoryPool( &ricTinyMemoryPool,
                    RIC_TINY_POOL_ID,
                    RIC_TINY_POOL_BLOCK_SIZE,
                    sizeof( RiCTinyMemoryBlock ),
                    RIC_TINY_POOL_BLOCKS,
                    ricTinyMemoryBlocks );
#endif

    /* Initialize the small memory pool. */
#if RIC_SMALL_POOL_BLOCKS > 0U
    initMemoryPool( &ricSmallMemoryPool,
                    RIC_SMALL_POOL_ID,
                    RIC_SMALL_POOL_BLOCK_SIZE,
                    sizeof( RiCSmallMemoryBlock ),
                    RIC_SMALL_POOL_BLOCKS,
                    ricSmallMemoryBlocks );
#endif

    /* Initialize the medium memory pool. */
#if RIC_MEDIUM_POOL_BLOCKS > 0U
    initMemoryPool( &ricMediumMemoryPool,
                    RIC_MEDIUM_POOL_ID,
                    RIC_MEDIUM_POOL_BLOCK_SIZE,
                    sizeof( RiCMediumMemoryBlock ),
                    RIC_MEDIUM_POOL_BLOCKS,
                    ricMediumMemoryBlocks );
#endif

    /* Initialize the large memory pool. */
#if RIC_LARGE_POOL_BLOCKS > 0U
    initMemoryPool( &ricLargeMemoryPool,
                    RIC_LARGE_POOL_ID,
                    RIC_LARGE_POOL_BLOCK_SIZE,
                    sizeof( RiCLargeMemoryBlock ),
                    RIC_LARGE_POOL_BLOCKS,
                    ricLargeMemoryBlocks );
#endif

    /* Initialize the huge memory pool. */
#if RIC_HUGE_POOL_BLOCKS > 0U
    initMemoryPool( &ricHugeMemoryPool,
                    RIC_HUGE_POOL_ID,
                    RIC_HUGE_POOL_BLOCK_SIZE,
                    sizeof( RiCHugeMemoryBlock ),
                    RIC_HUGE_POOL_BLOCKS,
                    ricHugeMemoryBlocks );
#endif
}

/*+***************************************************************************** 
 * Input:     requestedSize: The size of the memory request in bytes.
 *
 * Output:    RiCGetMemory(): The contents pointer to a block of memory from the
 *                            pool whose block size is closest to the requested
 *                            size or NULL on failure.
 *
 * InOut:     None.
 *
 * Function:  Using the requested size of memory, grab a block from the proper
 *            pool and return a pointer to its contents.  If the pool is empty,
 *            try to grab successively larger blocks from the other pools.
 *            Returns NULL if no pools can service the request.  This function
 *            can replace the standard "malloc" function.
 *
 *------------------------------------------------------------------------------
 * Author             Date        Modification
 * J.Collins,P.Urban  09.09.2005  Updates for MISRA-C and EADS coding standards
 *
 ****************************************************************************-*/
RhpAddress RiCGetMemory( const size_t requestedSize )
{
    RhpAddress result = NULL;

    /* Attempt to get a block from the pool whose block size is closest to the
       size of the request.  If the request should fail for that pool, keep
       trying to get the block from the larger pools. */

#if RIC_TINY_POOL_BLOCKS > 0U
    if( requestedSize <= RIC_TINY_POOL_BLOCK_SIZE )
    {
        result = getMemoryFromMemoryPool( &ricTinyMemoryPool );
    }
#endif

    if( result == NULL )
    {
#if RIC_SMALL_POOL_BLOCKS > 0U
        if( requestedSize <= RIC_SMALL_POOL_BLOCK_SIZE )
        {
            result = getMemoryFromMemoryPool( &ricSmallMemoryPool );
        }
#endif
        if( result == NULL )
        {
#if RIC_MEDIUM_POOL_BLOCKS > 0U
            if( requestedSize <= RIC_MEDIUM_POOL_BLOCK_SIZE )
            {
                result = getMemoryFromMemoryPool( &ricMediumMemoryPool );
            }
#endif
            if( result == NULL )
            {
#if RIC_LARGE_POOL_BLOCKS > 0U
               if( requestedSize <= RIC_LARGE_POOL_BLOCK_SIZE )
                {
                    result = getMemoryFromMemoryPool( &ricLargeMemoryPool );
                }
#endif
                if( result == NULL )
                {
#if RIC_HUGE_POOL_BLOCKS > 0U
                   if( requestedSize <= RIC_HUGE_POOL_BLOCK_SIZE )
                    {
                        result = getMemoryFromMemoryPool( &ricHugeMemoryPool );
                    }
#endif
                }
            }
        }
    }

    /* If we still haven't found a suitable block, then none of the memory pools
       can service the request. */

    if( result == NULL )
    {
#ifdef RIC_USE_RICOUTPUT
        RiC_NOTIFY_TO_ERROR( "RiCGetMemory Failed - No memory pool can service request." );
#endif
    }

#ifdef USE_ASSERT
	RiC_ASSERT(result != NULL);
#endif

    return result;
}


/*+*****************************************************************************
 * Input:     contents: The pointer to a block's contents to be returned to the
 *                      proper memory pool.
 *
 * Output:    None.
 *
 * InOut:     None.
 *
 * Function:  Determine the memory pool from which the contents originated by
 *            examining its header and return it to the proper pool.  This
 *            function can replace the standard "free" function.
 *
 *------------------------------------------------------------------------------
 * Author             Date        Modification
 * J.Collins,P.Urban  09.09.2005  Updates for MISRA-C and EADS coding standards
 *
 ****************************************************************************-*/
void RiCReturnMemory( const RhpAddress pContents )
{
    /* First, get the block header for the given contents pointer.  Then get the
       memory pool using the block header's memory pool ID.  Finally, return the
       block to its corresponding memory pool. */

    RiCMemoryBlockHeader *blockHeader = getMemoryBlockHeader( pContents );

    if( blockHeader != NULL )
    {
        RiCMemoryPool *memoryPool = getMemoryPool( blockHeader->memoryPoolId );

        if( memoryPool != NULL )
        {
            returnMemoryToMemoryPool( memoryPool, blockHeader );
        }
        else
        {
#ifdef RIC_USE_RICOUTPUT
            RiC_NOTIFY_TO_ERROR( "RiCReturnMemory Failed - Attempted double delete of block or corrupt block header." );
#endif
        }
    }
    else
    {
#ifdef RIC_USE_RICOUTPUT
        RiC_NOTIFY_TO_ERROR( "RiCReturnMemory Failed - Given NULL contents pointer." );
#endif
    }
}


/*+*****************************************************************************
 * Input:     currentSize: The current size of the memory block.  Note that this
 *                         parameter is not used by this memory manager, nor is
 *                         it required by the standard "realloc" function.
 *
 *            newSize: The new size for the memory block.
 *
 * Output:    ricRealloc(): Returns the contents pointer to the reallocated
 *                          block or NULL on failure.
 *
 * InOut:     contents: The contents pointer of the existing memory block, which
 *                      may be reused or released.
 *
 * Function:  Reallocates the given memory block to the new size.  The current
 *            implemetation of this function is focused on increasing the size
 *            of a block.  If the new size is smaller than the current size, no
 *            attempt is made to switch to a smaller block at this time.  If the
 *            user wishes to do this, they can allocate their own smaller block
 *            and perform a memcpy manually prior to releasing their existing
 *            block.  This function can replace the standard "realloc"
 *            function.
 *
 *------------------------------------------------------------------------------
 * Author             Date        Modification
 * J.Collins,P.Urban  09.09.2005  Updates for MISRA-C and EADS coding standards
 *
 ****************************************************************************-*/
RhpAddress ricRealloc( RhpAddress pContents,
                  const size_t currentSize,
                  const size_t newSize )
{
    RhpAddress result = NULL;

    /* First get the block header for the given contents pointer.  If the given
       contents pointer is NULL, just allocate the memory using RiCGetMemory.
       If that allocation fails, return NULL. */

    RiCMemoryBlockHeader *blockHeader = getMemoryBlockHeader( pContents );

    if( blockHeader == NULL )
    {
        result = RiCGetMemory( newSize );
    }

    /* Otherwise, if the new size is 0, just release the existing block and
       return NULL. */

    else if( newSize == 0U )
    {
        RiCReturnMemory( pContents );
    }

    /* Otherwise, if the new size requires a larger memory block, attempt
       to allocate a new block.  If the allocation fails, return NULL and leave
       given block untouched.  If the allocation is successful, copy the
       existing block into the new block and release the existing block.
       Otherwise, just reuse the existing block since it is large enough to
       hold the new size. */
    else
    {
        RiCMemoryPool *memoryPool = getMemoryPool( blockHeader->memoryPoolId );

        if( memoryPool != NULL )
        {
            if( currentSize <= (memoryPool->usableBlockSize) )
            {
                if( newSize > (memoryPool->usableBlockSize) )
                {
                    result = RiCGetMemory( newSize );
                    if( result != NULL)
                    {
                        (void)memcpy( result, pContents,
                                    memoryPool->usableBlockSize );
                        RiCReturnMemory( pContents );
                    }
                }
                else
                {
                    result = pContents;
                }
            }
            else
            {
#ifdef RIC_USE_RICOUTPUT
                RiC_NOTIFY_TO_ERROR( "ricRealloc Failed - Expected current size of block does not match actual block size." );
#endif
            }
        }
        else
        {
#ifdef RIC_USE_RICOUTPUT
            RiC_NOTIFY_TO_ERROR( "ricRealloc Failed - Attempted double delete of block or corrupt block header." );
#endif
        }
    }

#ifdef USE_ASSERT
	if( newSize != 0U )
		RiC_ASSERT(result != NULL);
#endif

    return result;
}


/*+*****************************************************************************
 * Input:     memoryPoolId: The ID of the memory pool to examine usage.
 *
 * Output:    usage: The RiCMemoryPoolUsage structure that is populated with the
 *                   statistics of the given memory pool.
 *
 * InOut:     None.
 *
 * Function:  Populate the given usage structure with the statistics of the
 *            given memory pool ID.
 *
 *------------------------------------------------------------------------------
 * Author             Date        Modification
 * J.Collins,P.Urban  09.09.2005  Updates for MISRA-C and EADS coding standards
 *
 ****************************************************************************-*/
void RiCMemoryManagerUsage( const RiC_UINT32 memPoolId,
                            RiCMemoryPoolUsage *usage )
{
#if RIC_TINY_POOL_BLOCKS > 0U
    if( memPoolId == RIC_TINY_POOL_ID )
    {
        getMemoryPoolUsage( &ricTinyMemoryPool, usage );
    }
    else
#endif
#if RIC_SMALL_POOL_BLOCKS > 0U
    if( memPoolId == RIC_SMALL_POOL_ID )
    {
        getMemoryPoolUsage( &ricSmallMemoryPool, usage );
    }
    else
#endif
#if RIC_MEDIUM_POOL_BLOCKS > 0U
    if( memPoolId == RIC_MEDIUM_POOL_ID )
    {
        getMemoryPoolUsage( &ricMediumMemoryPool, usage );
    }
    else
#endif
#if RIC_LARGE_POOL_BLOCKS > 0U
    if( memPoolId == RIC_LARGE_POOL_ID )
    {
        getMemoryPoolUsage( &ricLargeMemoryPool, usage );
    }
    else
#endif
#if RIC_HUGE_POOL_BLOCKS > 0U
    if( memPoolId == RIC_HUGE_POOL_ID )
    {
        getMemoryPoolUsage( &ricHugeMemoryPool, usage );
    }
    else
#endif
    {
        /* Do Nothing */
    }
}


/*+*****************************************************************************
 * Input:     invertedMemoryPoolId: The ID of the memory pool to retrieve.  The
 *                                  ID must originate from the header of an
 *                                  allocated block!  In other words, it must be
 *                                  an inverted memory pool ID.
 *
 * Output:    getMemoryPool(): Returns the memory pool matching the given ID or
 *                             NULL if the given ID is corrupted.
 *
 * InOut:     None.
 *
 * Function:  Returns the memory pool for the given inverted memory pool ID.
 *            Note that the memory pool IDs begin with "DEADBEF" followed by a
 *            unique number, so it is unlikely for an ID to be corrupted without
 *            detecting the problem.
 *
 *            IMPORTANT!!! This private function should only be called with
 *            memory pool ids that originate from allocated block headers.  This
 *            is because we invert the memory pool ids of allocated blocks to
 *            detect double deletes.  This function automatically inverts the
 *            given memory pool id back before looking for a match with one of
 *            the memory pools.
 *
 *------------------------------------------------------------------------------
 * Author             Date        Modification
 * J.Collins,P.Urban  09.09.2005  Updates for MISRA-C and EADS coding standards
 *
 ****************************************************************************-*/
STATIC RiCMemoryPool *getMemoryPool( const RiC_UINT32 invertedMemoryPoolId )
{
    RiCMemoryPool *result = NULL;

    /* We invert the block's memory pool id within its header while it
       is allocated to the user.  When the block is returned, it's
       memory pool id is inverted back to its original value.  This
       technique allows us to detect and ignore a double-delete attempt
       by the user. */
        
    RiC_UINT32 memPoolId = ~invertedMemoryPoolId;

#if RIC_TINY_POOL_BLOCKS > 0U
    if( memPoolId == RIC_TINY_POOL_ID )
    {
        result = &ricTinyMemoryPool;
    }
    else
#endif
#if RIC_SMALL_POOL_BLOCKS > 0U
    if( memPoolId == RIC_SMALL_POOL_ID )
    {
        result = &ricSmallMemoryPool;
    }
    else
#endif
#if RIC_MEDIUM_POOL_BLOCKS > 0U
    if( memPoolId == RIC_MEDIUM_POOL_ID )
    {
        result = &ricMediumMemoryPool;
    }
    else
#endif
#if RIC_LARGE_POOL_BLOCKS > 0U
    if( memPoolId == RIC_LARGE_POOL_ID )
    {
        result = &ricLargeMemoryPool;
    }
    else
#endif
#if RIC_HUGE_POOL_BLOCKS > 0U
    if( memPoolId == RIC_HUGE_POOL_ID )
    {
        result = &ricHugeMemoryPool;
    }
    else
#endif
    {
        /* Do Nothing */
    }

#ifdef USE_ASSERT
	RiC_ASSERT(result != NULL);
#endif

    return result;
}


/*+*****************************************************************************
 * Input:     contents: A pointer to the contents of a fixed-sized block.
 *
 * Output:    getMemoryBlockHeader(): Returns the block's header for the given
 *                                    block's contents pointer or NULL if the
 *                                    given contents pointer is NULL.
 *
 * InOut:     None.
 *
 * Function:  Returns a pointer to the block header for the given block
 *            contents pointer.  All blocks begin with the same header,
 *            regardless of their size.  Because the header ends flush with
 *            the block's contents, we can just convert the given contents
 *            pointer to a header pointer and subtract 1 to point to the
 *            actual header of the block.
 *
 *------------------------------------------------------------------------------
 * Author             Date        Modification
 * J.Collins,P.Urban  09.09.2005  Updates for MISRA-C and EADS coding standards
 *
 ****************************************************************************-*/
STATIC RiCMemoryBlockHeader *getMemoryBlockHeader( const RhpAddress pContents )
{
    RiCMemoryBlockHeader *result = NULL;

    if( pContents != NULL )
    {
        result = ((RiCMemoryBlockHeader*)pContents) - 1;
    }

    return result;
}


/*+*****************************************************************************
 * Input:     memoryPoolId: The unique ID of the memory pool.
 *
 *            usableBlockSize: The size of the block's contents returned to the
 *                             user from this memory pool.
 *
 *            totalBlockSize: The total size of a block in the pool, which
 *                            is the usabelBlockSize plus the block header size.
 *
 *            numBlocks: The number of blocks in the memory pool.
 *
 *            startOfPool: A pointer to the buffer that holds the blocks for
 *                         this pool.
 *
 * Output:    None.
 *
 * InOut:     me: The memory pool to initialize.
 *
 * Function:  Initializes a memory pool.
 *
 *------------------------------------------------------------------------------
 * Author             Date        Modification
 * J.Collins,P.Urban  09.09.2005  Updates for MISRA-C and EADS coding standards
 *
 ****************************************************************************-*/
STATIC void initMemoryPool( RiCMemoryPool *const me,
                            const RiC_UINT32 memPoolId,
                            const RiC_UINT32 usableBSize,
                            const RiC_UINT32 totalBSize,
                            const RiC_UINT32 nBlocks,
                            RhpAddress startPool )
{
    RiC_UINT32 blockCount;
    RiC_INT8 *currentBlock;
    RiCTinyMemoryBlock *block;

    if( (me != NULL) && (startPool != NULL) )
    {
        /* Initialize the various attributes of this memory pool using the
           given data. */

        me->memoryPoolId = memPoolId;
        me->usableBlockSize = usableBSize;
        me->totalBlockSize = totalBSize;
        me->numBlocks = nBlocks;
        me->blocksInUse = 0;
        me->maxBlocksInUse = 0;
        me->startOfPool = startPool;
        me->freeList = startPool;

        /* Initialize the mutex used to guard this memory pool against
           multi-threaded access. */

        (void)RiCOSMutex_init( &(me->memoryPoolMutex), (OSMutexHandle)0);

        /* Finally, we must initialize the blocks within the memory pool.  This
           is accomplished by configuring the header of each block with the
           memory pool ID and configuring the contents of each block with the
           pointer to the next block in the pool.

           Note regarding lint !e826 comments below: Because each block is of a
           different size in each pool, we use a simple byte pointer called
           "currentBlock" to position ourself within the pool.  This pointer is
           typed to a tiny memory block to actually configure the block header
           and next pointer.  Because all blocks share the same header which is
           flush to its contents field, this approach works fine and lets us
           use the same code to initialize all memory pools. */

        currentBlock = startPool;

        for( blockCount = 0; blockCount < (me->numBlocks); blockCount++ )
        {
            block = (RiCTinyMemoryBlock*)currentBlock; /*lint !e826 */
            block->header.memoryPoolId = memPoolId;

            if( blockCount < (me->numBlocks - 1U) )
            {
                currentBlock = currentBlock + totalBSize;
                block->contents.next = (RiCTinyMemoryBlock*)
                                       currentBlock; /*lint !e826 */
            }
            else
            {
                block->contents.next = NULL;
            }
        }
    }
}


/*+*****************************************************************************
 * Input:     me: The memory pool to access.
 *
 * Output:    checkIfMemoryPoolIsEmpty(): Returns true if the memory pool is
 *                                        empty.
 *
 * InOut:     None.
 *
 * Function:  Checks if a memory pool is empty.
 *
 *------------------------------------------------------------------------------
 * Author             Date        Modification
 * J.Collins,P.Urban  09.09.2005  Updates for MISRA-C and EADS coding standards
 *
 ****************************************************************************-*/
STATIC RiCBoolean checkIfMemoryPoolIsEmpty( const RiCMemoryPool *const me )
{
    RiCBoolean result = RiCFALSE;

    if( me != NULL )
    {
        if( (me->freeList) == NULL )
        {
            result = RiCTRUE;
        }
    }

    return result;
}


/*+*****************************************************************************
 * Input:     None.
 *
 * Output:    getMemoryFromMemoryPool(): Return a block of memory from the pool
 *                                       or NULL on failure.
 *
 * InOut:     me: The memory pool to provide the block.
 *
 * Function:  Grab a block of memory from a pool.
 *
 *------------------------------------------------------------------------------
 * Author             Date        Modification
 * J.Collins,P.Urban  09.09.2005  Updates for MISRA-C and EADS coding standards
 *
 ****************************************************************************-*/
STATIC RhpAddress getMemoryFromMemoryPool( RiCMemoryPool *const me )
{
    RhpAddress result = NULL;
    RiCTinyMemoryBlock *block;

    if( me != NULL )
    {
        (void)RiCOSMutex_lock( &(me->memoryPoolMutex) );

        if(( !checkIfMemoryPoolIsEmpty( me ) ) && ((me->freeList) != NULL) )
        {
            /* Because all blocks begin with the same header that is flush to
               its contents, we can type any sized block as a tiny block to
               reuse this code for all memory pools.  Simply get the next
               block off the free list and update the free list to point to
               the next block in the free pool.  Bump up the number of blocks
               in use and bump up the high water mark if necessary.  The
               pointer returned to the user is the contents portion of the
               block.  The header is not returned to the user. */

            block = (RiCTinyMemoryBlock*)me->freeList;
            me->freeList = block->contents.next;
            me->blocksInUse++;
            if( me->blocksInUse > (me->maxBlocksInUse) )
            {
                me->maxBlocksInUse = me->blocksInUse;
            }
            result = block->contents.memory;

            /* We invert the block's memory pool id within its header while it
               is allocated to the user.  When the block is returned, it's
               memory pool id is inverted back to its original value.  This
               technique allows us to detect and ignore a double-delete attempt
               by the user. */
               
            block->header.memoryPoolId = ~(block->header.memoryPoolId);
        }

        (void)RiCOSMutex_free( &(me->memoryPoolMutex) );
    }

#ifdef USE_ASSERT
	RiC_ASSERT(result != NULL);
#endif

    return result;
}


/*+*****************************************************************************
 * Input:     memory: The block to return to the pool.
 *
 * Output:    None.
 *
 * InOut:     me: The memory pool to return the block.
 *
 * Function:  Return a block of memory to a pool.
 *
 *------------------------------------------------------------------------------
 * Author             Date        Modification
 * J.Collins,P.Urban  09.09.2005  Updates for MISRA-C and EADS coding standards
 *
 ****************************************************************************-*/
STATIC void returnMemoryToMemoryPool( RiCMemoryPool *const me,
                                      RhpAddress blockHeader )
{
    RiCTinyMemoryBlock *block;

    if( (me != NULL) && (blockHeader != NULL) )
    {
        (void)RiCOSMutex_lock( &(me->memoryPoolMutex) );

        /* Because all blocks begin with the same header that is flush with its
           contents, we can type any sized block as a tiny block to reuse this
           code for all memory pools.  Simply set the block's next pointer to
           the current free list pointer and then update the free list to point
           to this block.  Finally, decrement the number of blocks in use. */

        block = blockHeader;
        block->contents.next = me->freeList;
        me->freeList = block;
        me->blocksInUse--;

        /* We invert the block's memory pool id within its header while it
            is allocated to the user.  When the block is returned, it's
            memory pool id is inverted back to its original value.  This
            technique allows us to detect and ignore a double-delete attempt
            by the user. */
            
        block->header.memoryPoolId = ~(block->header.memoryPoolId);

        (void)RiCOSMutex_free( &(me->memoryPoolMutex) );
    }
}


/*+*****************************************************************************
 * Input:     me: The memory pool to analyze.
 *
 * Output:    validateMemoryPool(): Return RiCTRUE if valid.
 *
 * InOut:     none.
 *
 * Function:  Validate that the memory pool has not been corrupted.
 *
 *------------------------------------------------------------------------------
 * Author             Date        Modification
 * J.Collins,P.Urban  09.09.2005  Updates for MISRA-C and EADS coding standards
 *
 ****************************************************************************-*/
STATIC RiCBoolean validateMemoryPool( const RiCMemoryPool *const me )
{
    RiCBoolean result = RiCTRUE;
    RiC_UINT32 blockCount;
    RiC_INT8 *currentBlock;
    RiCTinyMemoryBlock *block;

    if( me != NULL )
    {
        /* First walk through all the blocks verifying that they have the
           correct memory pool ID, regardless of whether they have been
           allocated or not.

           Note regarding lint !e826 comment below: Because each block is of a
           different size in each pool, we use a simple byte pointer called
           "currentBlock" to position ourself within the pool.  This pointer is
           typed to a tiny memory block to actually analyze the block header.
           Because all blocks share the same header which is flush to its
           contents field, this approach works fine and lets us use the same
           code to analyze all memory pools. */

        currentBlock = me->startOfPool;

        for( blockCount = 0; ((blockCount < (me->numBlocks)) && (result));
             blockCount++ )
        {
            block = (RiCTinyMemoryBlock*)currentBlock; /*lint !e826 */
            if( (block != NULL) &&
				(block->header.memoryPoolId != (me->memoryPoolId)) &&
                (block->header.memoryPoolId != ~(me->memoryPoolId)) )
            {
                result = RiCFALSE;
            }
            else
            {
                currentBlock = currentBlock + me->totalBlockSize;
            }
        }

        /* If we passed the first test, now walk through all of the blocks in
           the free list ensuring they point to valid blocks. */

        if( result )
        {
            block = me->freeList;

            for( blockCount = 0;
                ((blockCount < (me->numBlocks - me->blocksInUse)) && (result) && (block != NULL));
                blockCount++ )
            {
                if( block->header.memoryPoolId != (me->memoryPoolId) )
                {
                    result = RiCFALSE;
                }
                else
                {
                    block = block->contents.next;
                }
            }
        }

    }

    return result;
}


/*+*****************************************************************************
 * Input:     me: The memory pool to analyze.
 *
 * Output:    usage: The RiCMemoryPoolUsage structure that is populated with the
 *                   statistics of the given memory pool.
 *
 * InOut:     none.
 *
 * Function:  Populate the given usage structure with the statistics of the
 *            given memory pool.
 *
 *------------------------------------------------------------------------------
 * Author             Date        Modification
 * J.Collins,P.Urban  09.09.2005  Updates for MISRA-C and EADS coding standards
 *
 ****************************************************************************-*/
STATIC void getMemoryPoolUsage( RiCMemoryPool *const me,
                                RiCMemoryPoolUsage *usage )
{
    if(( me != NULL ) && ( usage != NULL ))
    {
        (void)RiCOSMutex_lock( &(me->memoryPoolMutex) );

        usage->memoryPoolId = me->memoryPoolId;
        usage->usableBlockSize = me->usableBlockSize;
        usage->totalBlockSize = me->totalBlockSize;
        usage->numBlocks = me->numBlocks;
        usage->blocksInUse = me->blocksInUse;
        usage->maxBlocksInUse = me->maxBlocksInUse;
        usage->valid = validateMemoryPool( me );

        (void)RiCOSMutex_free( &(me->memoryPoolMutex) );
    }
}

#endif

