#ifndef RICMEMORYMANAGER_H
#define RICMEMORYMANAGER_H

/*

	Licensed Materials - Property of IBM
	(c) Copyright IBM Corporation 2006, 2011. All Rights Reserved.
*/

/*+*****************************************************************************
********************************************************************************
** Project:     Rhapsody in C Framework
**
** Author:      Jim Collins, Justin Dyer, Mark Richardson, 01.01.2005
**
** Language:    C
**
** Processor:   N/A
**
** Module Name: RiCMemoryManager.h
**
** Function:    A memory manager based upon multiple pools of fixed-sized
**              blocks that provides deterministic memory allocation and avoids
**              fragmentation at the expense of some "wasted" memory.
**              
**              There are five memory pools named after the size of the blocks
**              they contain: Tiny, Small, Medium, Large, and Huge.  The size
**              of the blocks in each pool and the number of blocks in each
**              pool can be customized and "tuned" for each application.  Also,
**              the alignment of blocks can be configured to match the most
**              restrictive alignment type for the given processor/compiler.
**
**              The RiCInitMemoryManager function must be invoked when the
**              application is first started to initialize the memory pools.
**              This must be done prior to any memory allocation.  The malloc
**              function can be replaced with the RiCGetMemory function.  The
**              free function can be replaced with the RiCReturnMemory function.
**              The realloc function can be replaced with the ricRealloc
**              function.  Finally, the RiCMemoryManagerUsage function can be
**              invoked to examine the usage of each memory pool and to check
**              whether each pool has been corrupted.
**              
**------------------------------------------------------------------------------
** Generator:     N/A
** Component:     N/A
** Configuration: N/A
** Model Element: N/A
********************************************************************************
*****************************************************************************-*/

/* Includes *******************************************************************/

#include "mxf_cfg.h"
#include "RiCOSWrap.h"

/* Constants ******************************************************************/


/* The unique IDs of the fixed-sized memory pools.  Each block is prefixed with
   a header that contains the block's corresponding memory pool ID.  This ID is
   used to return the block back into the proper memory pool.  These IDs are
   also used to detect memory pool corruptions, which is why each ID begins with
   "DEADBEF" followed by a numeric identifier 1-5. */

#define RIC_TINY_POOL_ID 0xDEADBEF1U
#define RIC_SMALL_POOL_ID 0xDEADBEF2U
#define RIC_MEDIUM_POOL_ID 0xDEADBEF3U
#define RIC_LARGE_POOL_ID 0xDEADBEF4U
#define RIC_HUGE_POOL_ID 0xDEADBEF5U

/* The block sizes of each of the memory pools.  These block sizes must be
   incrementally larger and must also be a multiple of the size of the most
   restrictive alignment type for the processor/compiler.  The smallest block
   size must be capable of holding a "next" pointer that links blocks in the
   free list within each memory pool.  The memory utilization of the
   application should be profiled to determine the optimal sizes for the fixed-
   sized blocks to minimize wasted memory. */

#define RIC_TINY_POOL_BLOCK_SIZE 32U
#define RIC_SMALL_POOL_BLOCK_SIZE 256U
#define RIC_MEDIUM_POOL_BLOCK_SIZE 1024U
#define RIC_LARGE_POOL_BLOCK_SIZE 8192U
#define RIC_HUGE_POOL_BLOCK_SIZE 16384U

/* The size of the message buffer when reporting pool usage and status. */

#define RIC_POOL_USAGE_BUF_SIZE 256

/* Macros *********************************************************************/

/* Types **********************************************************************/

/* This typedef must be set to the most restrictive alignment type for the given
   processor/compiler.  The memory blocks are aligned using this type to ensure
   correct and efficient operation, regardless of the type actually stored in
   the block. */

typedef RiC_INT32 RiCMostRestrictiveAlignmentType;

/* Each block stored in a memory pool is prefixed with the following common
   header.  It contains the unique id of the memory pool from which the block
   originated.  The header is aligned using the most restrictive type to ensure
   that the block's contents immediately follow the header without any
   padding. */

/* Justification: Violation of Misra98 Rule110 / Misra04 Rule18.4 */
/* union is required when using the memory allocation utility	  */
typedef union RiCMemoryBlockHeaderTag
{
    RiC_UINT32 memoryPoolId;
    RiCMostRestrictiveAlignmentType align;
} RiCMemoryBlockHeader;

/* A block stored in the tiny memory pool.  The block is split into two
   sections: header and contents.  The header was described above.  The
   contents depend upon whether the block is allocated or not.  When the block
   is unallocated (in the free list), the contents hold the pointer to the next
   block in the free list.  When the block is allocated, the contents hold the
   user's data.  The contents are aligned using the most restrictive type to
   ensure correct and efficient operation, regardless of the type actually
   stored in the block. */

typedef struct RiCTinyMemoryBlock_t
{
    RiCMemoryBlockHeader header;
	/* Justification: Violation of Misra98 Rule110 / Misra04 Rule18.4 */
	/* union is required when using the memory allocation utility	  */
    union RiCTinyMemoryBlockContents
    {
        struct RiCTinyMemoryBlock_t *next;
        RiC_UINT8 memory[RIC_TINY_POOL_BLOCK_SIZE];
        RiCMostRestrictiveAlignmentType align;
    } contents;
} RiCTinyMemoryBlock;

/* A block stored in the small memory pool. */

typedef struct RiCSmallMemoryBlock_t
{
    RiCMemoryBlockHeader header;
	/* Justification: Violation of Misra98 Rule110 / Misra04 Rule18.4 */
	/* union is required when using the memory allocation utility	  */
    union RiCSmallMemoryBlockContents
    {
        struct RiCSmallMemoryBlock_t *next;
        RiC_UINT8 memory[RIC_SMALL_POOL_BLOCK_SIZE];
        RiCMostRestrictiveAlignmentType align;
    } contents;
} RiCSmallMemoryBlock;

/* A block stored in the medium memory pool. */

typedef struct RiCMediumMemoryBlock_t
{
    RiCMemoryBlockHeader header;
	/* Justification: Violation of Misra98 Rule110 / Misra04 Rule18.4 */
	/* union is required when using the memory allocation utility	  */
    union RiCMediumMemoryBlockContents
    {
        struct RiCMediumMemoryBlock_t *next;
        RiC_UINT8 memory[RIC_MEDIUM_POOL_BLOCK_SIZE];
        RiCMostRestrictiveAlignmentType align;
    } contents;
} RiCMediumMemoryBlock;

/* A block stored in the large memory pool. */

typedef struct RiCLargeMemoryBlock_t
{
    RiCMemoryBlockHeader header;
	/* Justification: Violation of Misra98 Rule110 / Misra04 Rule18.4 */
	/* union is required when using the memory allocation utility	  */
    union RiCLargeMemoryBlockContents
    {
        struct RiCLargeMemoryBlock_t *next;
        RiC_UINT8 memory[RIC_LARGE_POOL_BLOCK_SIZE];
        RiCMostRestrictiveAlignmentType align;
    } contents;
} RiCLargeMemoryBlock;

/* A block stored in the huge memory pool. */

typedef struct RiCHugeMemoryBlock_t
{
    RiCMemoryBlockHeader header;
	/* Justification: Violation of Misra98 Rule110 / Misra04 Rule18.4 */
	/* union is required when using the memory allocation utility	  */
    union RiCHugeMemoryBlockContents
    {
        struct RiCHugeMemoryBlock_t *next;
        RiC_UINT8 memory[RIC_HUGE_POOL_BLOCK_SIZE];
        RiCMostRestrictiveAlignmentType align;
    } contents;
} RiCHugeMemoryBlock;

/* Each memory pool is identified with a unique ID.  It also holds the usable
   size of each block in the pool and the total size of each block including its
   header.  The memory pool structure holds the total number of blocks in the
   pool.  Each pool also maintains two statistics that can be used for profiling
   the application: the current number of blocks in use and the maximum number
   of blocks in use at any given time (the high water mark).  A mutex guards the
   memory pool for multi-threaded access.  Finally, each memory pool maintains
   pointers to the start of the buffer holding its blocks and to the start of
   its list of free blocks. */

typedef struct RiCMemoryPool_t
{
    RiC_UINT32 memoryPoolId;
    RiC_UINT32 usableBlockSize;
    RiC_UINT32 totalBlockSize;
    RiC_UINT32 numBlocks;
    RiC_UINT32 blocksInUse;
    RiC_UINT32 maxBlocksInUse;
    RiCOSMutex memoryPoolMutex;
    RhpAddress startOfPool;
    RhpAddress freeList;
} RiCMemoryPool;

/* The structure that is populated with the results from the call to
   RiCMemoryManagerUsage. */

typedef struct RiCMemoryPoolUsage_t
{
    RiC_UINT32 memoryPoolId;
    RiC_UINT32 usableBlockSize;
    RiC_UINT32 totalBlockSize;
    RiC_UINT32 numBlocks;
    RiC_UINT32 blocksInUse;
    RiC_UINT32 maxBlocksInUse;
    RiCBoolean valid;
} RiCMemoryPoolUsage;



/* Prototypes *****************************************************************/

void RiCInitMemoryManager( void );

RhpAddress RiCGetMemory( const size_t requestedSize );

void RiCReturnMemory( const RhpAddress pContents );

void RiCMemoryManagerUsage( const RiC_UINT32 memPoolId,
                            RiCMemoryPoolUsage *usage );



#endif
