/*

	Licensed Materials - Property of IBM
	(c) Copyright IBM Corporation 2006, 2013. All Rights Reserved.
*/

/********************************************************************
 ---------------------------------------------------------------
|							  RiCQueue.c                          |
 --------------------------------------------------------------- 
*********************************************************************/
#ifndef MXF_SOURCE
#define MXF_SOURCE
#endif
#ifndef MXF_RICQUEUE_SOURCE
#define MXF_RICQUEUE_SOURCE
#endif

#include <stdlib.h>
#include "RiCQueue.h"
#ifdef RIC_USE_RICQUEUE

/***************************************************/
/****			 Private operations		        ****/
/***************************************************/
static RiCBoolean isFull_(const RiCQueue * const me);
static void increaseHead_(RiCQueue * const me);
static void increaseTail_(RiCQueue * const me);

/** Internal check of Queue occupation status **/
static RiCBoolean isFull_(const RiCQueue * const me)
{
	RiCBoolean res = RiCTRUE;
	if (me != NULL)
	{
		if (me->count != me->size){
			res = RiCFALSE;
		}
	}
	return res;
}

/** Move the queue head to a new position, after removing the top **/
static void increaseHead_(RiCQueue * const me)
{
	if (me != NULL)
	{
		if (RiCQueue_isEmpty(me) == RiCFALSE){
			(me->head)++;
			if ((me->head) == (me->size)){
				me->head = 0U;
			}
		}
	}
}

/** Move the queue tail after adding a new element **/
static void increaseTail_(RiCQueue * const me)
{
	/* handle full queue */
	if (me != NULL)
	{
		if (isFull_(me) == RiCFALSE) {
			/* queue not empty */
			++(me->tail);
			if ((me->tail == me->size) && (me->head > 0U)){
				me->tail = 0U;
			}
		}
	}
}



/***************************************************/
/****          Creation and Destruction         ****/
/***************************************************/
RiCBoolean RiCQueue_Init(RiCQueue * const me, gen_ptr * const evQueue, RhpPositive queueSize)
{
	RiCBoolean retVal = RiCFALSE;
	if (me != NULL)
	{
		me->head = 0U;
		me->tail = 0U;
		me->count = 0U;
		me->size = queueSize;
		me->the_queue = evQueue;
		retVal = RiCTRUE;
	}
	return retVal;
}

/*******************/
/*******************/
#ifdef MICROC_OXF_CLEANUP
void RiCQueue_Cleanup(RiCQueue * const me)
{
	if (me != NULL)
	{
		me->head = 0;
		me->tail = 0;
		me->count = 0;
	}
}
#endif

/*******************/
/*******************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
RiCQueue * RiCQueue_Create(void)
{
	/* Justification: Violation of Misra98 Rule45 / Misra04 Rule11.4				*/
	/* Cast from void* pointer is required when using the memory allocation utility */
	/*LDRA_INSPECTED 94 S */
	/*LDRA_INSPECTED 95 S */ /*LDRA_INSPECTED 203 S */
	RiCQueue * me = (RiCQueue *) RiC_malloc((RhpPositive)sizeof(RiCQueue));
	if (me != NULL){
		/*LDRA_INSPECTED 45 D : LDRA Analysis incorrect. Pointer NULL cannot be checked for null */
		(void) RiCQueue_Init(me, (gen_ptr *) NULL, 0U);
	}
	/*LDRA_INSPECTED 71 S */

#ifdef USE_ASSERT
	RiC_ASSERT(me != NULL);
#endif

	return me;
}

/*******************/
/*******************/
#ifdef MICROC_OXF_CLEANUP
void RiCQueue_Destroy(RiCQueue * const me)
{
	if (me != NULL) {
		RiCQueue_Cleanup(me);
		RiC_free(me);
	}
}
#endif


/***************************************************/
/****            Queue methods             ****/
/***************************************************/

RiCBoolean RiCQueue_push(RiCQueue * const me, gen_ptr const pItem)
{
	RiCBoolean res = RiCFALSE;
	if (me != NULL)
	{
		if (isFull_(me) == RiCFALSE){
			me->the_queue[me->tail] = pItem;
			increaseTail_(me);
			me->count++;
			res = RiCTRUE;
		}
	}
#ifdef USE_ASSERT
	RiC_ASSERT(res != RiCFALSE);
#endif

	return res;
}

/*******************/
/*******************/
gen_ptr RiCQueue_pop(RiCQueue * const me)
{
	gen_ptr res = NULL;
	gen_ptr tmp;

	if (me != NULL)
	{
		if (RiCQueue_isEmpty(me) == RiCFALSE){
			tmp = me->the_queue[me->head];
			increaseHead_(me);
			me->count--;
			if(tmp != NULL){
				res = tmp;
			}
		}
	}

#ifdef USE_ASSERT
	RiC_ASSERT(res != NULL);
#endif

	/*LDRA_INSPECTED 71 S */
	return res;
}

#ifdef RIC_USE_RICLIST
/*******************/
/*******************/
void RiCQueue_getQueue(const RiCQueue * const me, RiCList * const pList)
{
	RhpPositive last, j;

	/* reset the list */
	if (me != NULL)
	{
		RiCList_removeAll(pList);
		if (RiCQueue_isEmpty(me) == RiCFALSE){
			last = (me->tail > me->head) ? me->tail : me->size;

			/* copy first part */
			j = me->head;
			for (; j < last; j++) {
				if (me->the_queue[j] != NULL){
					(void) RiCList_addTail(pList, me->the_queue[j]);
				}
			}

			/* check if need to continue */
			if (last != me->tail){
				/* copy second part */
				j = 0U;
				for (; j < me->tail; j++) {
					(void) RiCList_addTail(pList, me->the_queue[j]);
				}
			}
		}
	}
}
#endif

/*******************/
/*******************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
RhpPositive RiCQueue_getSize(const RiCQueue * const me)
{
	RhpPositive retVal = 0U;
	if (me != NULL)
		retVal = me->size;

	return retVal;
}

/*******************/
/*******************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
RhpPositive RiCQueue_getCount(const RiCQueue * const me)
{
	RhpPositive retVal = 0U;
	if (me != NULL)
		retVal = me->count;

	return retVal;
}

/*******************/
/*******************/
RiCBoolean RiCQueue_isEmpty(const RiCQueue * const me)
{
	RiCBoolean res = RiCTRUE;
	if (me != NULL)
	{
		if (me->count != 0U){
			res = RiCFALSE;
		}
	}

	return res;
}

/*******************/
/*******************/
RiCBoolean RiCQueue_isFull(const RiCQueue * const me)
{
	return isFull_(me);
}

/***************************************************/
/****            Iteration methods              ****/
/***************************************************/

/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
gen_ptr RiCQueue_first(const RiCQueue * const me, RhpPositive * pPos)
{
	gen_ptr res = NULL;
	if (me != NULL)
	{
		if ((RiCQueue_isEmpty(me) == RiCFALSE) && (pPos != NULL)){
			*pPos = me->head;
			/*LDRA_INSPECTED 45 D : LDRA Analysis incorrect. Pointer is checked for null before use */
			res = me->the_queue[*pPos];
		}
	}
#ifdef USE_ASSERT
	RiC_ASSERT(res != NULL);
#endif


	/*LDRA_INSPECTED 71 S */
	return res;
}

/*******************/
/*******************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
gen_ptr RiCQueue_next(const RiCQueue * const me, RhpPositive * pPos)
{
	gen_ptr res = NULL;
	if (me != NULL)
	{
		if (RiCQueue_isEmpty(me) == RiCFALSE){
			(*pPos)++;
			
			if (*pPos != me->tail) {
				if (*pPos == me->size){
					*pPos = 0U;
				}
				res = me->the_queue[*pPos];
			}
#ifdef USE_ASSERT
			else /* beyond end of queue */
			{
				RiC_ASSERT(NULL);
			}
#endif
		}
	}
#ifdef USE_ASSERT
	/* queue is empty or does not exist */
	RiC_ASSERT(res != NULL);
#endif

	/*LDRA_INSPECTED 71 S */
	return res;
}

/*******************/
/*******************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
gen_ptr RiCQueue_get(const RiCQueue * const me, RhpPositive pPos)
{
	gen_ptr res = NULL;
	if (me != NULL)
	{
		if (RiCQueue_isEmpty(me) == RiCFALSE){
			res = me->the_queue[pPos];
		}
	}
#ifdef USE_ASSERT
	RiC_ASSERT(res != NULL);
#endif


	/*LDRA_INSPECTED 71 S */
	return res;
}

/*******************/
/*******************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
RiCBoolean RiCQueue_isDone(const RiCQueue * const me, RhpPositive pPos)
{
	RiCBoolean res = RiCTRUE;
	if (me != NULL)
	{
		if (RiCQueue_isEmpty(me) == RiCFALSE){
			if (pPos != me->tail){
				if ((pPos < me->size) || (me->tail != 0U)){
					res = RiCFALSE;
				}
			}
		}
	}

	return res;
}


void RiCQueue_setFixedSize(const RiCQueue * const me, RiCBoolean b)
{
}

#endif
